# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class FirehoseDeliveryStream(pulumi.CustomResource):
    arn: pulumi.Output[str]
    """
    The Amazon Resource Name (ARN) specifying the Stream
    """
    destination: pulumi.Output[str]
    """
    This is the destination to where the data is delivered. The only options are `s3` (Deprecated, use `extended_s3` instead), `extended_s3`, `redshift`, `elasticsearch`, and `splunk`.
    """
    destination_id: pulumi.Output[str]
    elasticsearch_configuration: pulumi.Output[dict]
    """
    Configuration options if elasticsearch is the destination. More details are given below.

      * `bufferingInterval` (`float`) - Buffer incoming data for the specified period of time, in seconds between 60 to 900, before delivering it to the destination.  The default value is 300s.
      * `bufferingSize` (`float`) - Buffer incoming data to the specified size, in MBs between 1 to 100, before delivering it to the destination.  The default value is 5MB.
      * `cloudwatch_logging_options` (`dict`) - The CloudWatch Logging Options for the delivery stream. More details are given below
        * `enabled` (`bool`) - Enables or disables the logging. Defaults to `false`.
        * `log_group_name` (`str`) - The CloudWatch group name for logging. This value is required if `enabled` is true.
        * `logStreamName` (`str`) - The CloudWatch log stream name for logging. This value is required if `enabled` is true.

      * `domainArn` (`str`) - The ARN of the Amazon ES domain.  The IAM role must have permission for `DescribeElasticsearchDomain`, `DescribeElasticsearchDomains`, and `DescribeElasticsearchDomainConfig` after assuming `RoleARN`.  The pattern needs to be `arn:.*`.
      * `indexName` (`str`) - The Elasticsearch index name.
      * `indexRotationPeriod` (`str`) - The Elasticsearch index rotation period.  Index rotation appends a timestamp to the IndexName to facilitate expiration of old data.  Valid values are `NoRotation`, `OneHour`, `OneDay`, `OneWeek`, and `OneMonth`.  The default value is `OneDay`.
      * `processingConfiguration` (`dict`) - The data processing configuration.  More details are given below.
        * `enabled` (`bool`) - Enables or disables data processing.
        * `processors` (`list`) - Array of data processors. More details are given below
          * `parameters` (`list`) - Array of processor parameters. More details are given below
            * `parameterName` (`str`) - Parameter name. Valid Values: `LambdaArn`, `NumberOfRetries`, `RoleArn`, `BufferSizeInMBs`, `BufferIntervalInSeconds`
            * `parameterValue` (`str`) - Parameter value. Must be between 1 and 512 length (inclusive). When providing a Lambda ARN, you should specify the resource version as well.

          * `type` (`str`) - The type of processor. Valid Values: `Lambda`

      * `retryDuration` (`float`) - After an initial failure to deliver to Amazon Elasticsearch, the total amount of time, in seconds between 0 to 7200, during which Firehose re-attempts delivery (including the first attempt).  After this time has elapsed, the failed documents are written to Amazon S3.  The default value is 300s.  There will be no retry if the value is 0.
      * `role_arn` (`str`) - The ARN of the IAM role to be assumed by Firehose for calling the Amazon ES Configuration API and for indexing documents.  The pattern needs to be `arn:.*`.
      * `s3BackupMode` (`str`) - Defines how documents should be delivered to Amazon S3.  Valid values are `FailedDocumentsOnly` and `AllDocuments`.  Default value is `FailedDocumentsOnly`.
      * `typeName` (`str`) - The Elasticsearch type name with maximum length of 100 characters.
    """
    extended_s3_configuration: pulumi.Output[dict]
    """
    Enhanced configuration options for the s3 destination. More details are given below.

      * `bucketArn` (`str`) - The ARN of the S3 bucket
      * `bufferInterval` (`float`) - Buffer incoming data for the specified period of time, in seconds, before delivering it to the destination. The default value is 300.
      * `bufferSize` (`float`) - Buffer incoming data to the specified size, in MBs, before delivering it to the destination. The default value is 5.
        We recommend setting SizeInMBs to a value greater than the amount of data you typically ingest into the delivery stream in 10 seconds. For example, if you typically ingest data at 1 MB/sec set SizeInMBs to be 10 MB or higher.
      * `cloudwatch_logging_options` (`dict`) - The CloudWatch Logging Options for the delivery stream. More details are given below
        * `enabled` (`bool`) - Enables or disables the logging. Defaults to `false`.
        * `log_group_name` (`str`) - The CloudWatch group name for logging. This value is required if `enabled` is true.
        * `logStreamName` (`str`) - The CloudWatch log stream name for logging. This value is required if `enabled` is true.

      * `compressionFormat` (`str`) - The compression format. If no value is specified, the default is UNCOMPRESSED. Other supported values are GZIP, ZIP & Snappy. If the destination is redshift you cannot use ZIP or Snappy.
      * `dataFormatConversionConfiguration` (`dict`) - Nested argument for the serializer, deserializer, and schema for converting data from the JSON format to the Parquet or ORC format before writing it to Amazon S3. More details given below.
        * `enabled` (`bool`) - Defaults to `true`. Set it to `false` if you want to disable format conversion while preserving the configuration details.
        * `inputFormatConfiguration` (`dict`) - Nested argument that specifies the deserializer that you want Kinesis Data Firehose to use to convert the format of your data from JSON. More details below.
          * `deserializer` (`dict`) - Nested argument that specifies which deserializer to use. You can choose either the Apache Hive JSON SerDe or the OpenX JSON SerDe. More details below.
            * `hiveJsonSerDe` (`dict`) - Nested argument that specifies the native Hive / HCatalog JsonSerDe. More details below.
              * `timestampFormats` (`list`) - A list of how you want Kinesis Data Firehose to parse the date and time stamps that may be present in your input data JSON. To specify these format strings, follow the pattern syntax of JodaTime's DateTimeFormat format strings. For more information, see [Class DateTimeFormat](https://www.joda.org/joda-time/apidocs/org/joda/time/format/DateTimeFormat.html). You can also use the special value millis to parse time stamps in epoch milliseconds. If you don't specify a format, Kinesis Data Firehose uses java.sql.Timestamp::valueOf by default.

            * `openXJsonSerDe` (`dict`) - Nested argument that specifies the OpenX SerDe. More details below.
              * `caseInsensitive` (`bool`) - When set to true, which is the default, Kinesis Data Firehose converts JSON keys to lowercase before deserializing them.
              * `columnToJsonKeyMappings` (`dict`) - A map of column names to JSON keys that aren't identical to the column names. This is useful when the JSON contains keys that are Hive keywords. For example, timestamp is a Hive keyword. If you have a JSON key named timestamp, set this parameter to `{ ts = "timestamp" }` to map this key to a column named ts.
              * `convertDotsInJsonKeysToUnderscores` (`bool`) - When set to `true`, specifies that the names of the keys include dots and that you want Kinesis Data Firehose to replace them with underscores. This is useful because Apache Hive does not allow dots in column names. For example, if the JSON contains a key whose name is "a.b", you can define the column name to be "a_b" when using this option. Defaults to `false`.

        * `outputFormatConfiguration` (`dict`) - Nested argument that specifies the serializer that you want Kinesis Data Firehose to use to convert the format of your data to the Parquet or ORC format. More details below.
          * `serializer` (`dict`) - Nested argument that specifies which serializer to use. You can choose either the ORC SerDe or the Parquet SerDe. More details below.
            * `orcSerDe` (`dict`) - Nested argument that specifies converting data to the ORC format before storing it in Amazon S3. For more information, see [Apache ORC](https://orc.apache.org/docs/). More details below.
              * `blockSizeBytes` (`float`) - The Hadoop Distributed File System (HDFS) block size. This is useful if you intend to copy the data from Amazon S3 to HDFS before querying. The default is 256 MiB and the minimum is 64 MiB. Kinesis Data Firehose uses this value for padding calculations.
              * `bloomFilterColumns` (`list`) - A list of column names for which you want Kinesis Data Firehose to create bloom filters.
              * `bloomFilterFalsePositiveProbability` (`float`) - The Bloom filter false positive probability (FPP). The lower the FPP, the bigger the Bloom filter. The default value is `0.05`, the minimum is `0`, and the maximum is `1`.
              * `compression` (`str`) - The compression code to use over data blocks. The possible values are `UNCOMPRESSED`, `SNAPPY`, and `GZIP`, with the default being `SNAPPY`. Use `SNAPPY` for higher decompression speed. Use `GZIP` if the compression ratio is more important than speed.
              * `dictionaryKeyThreshold` (`float`) - A float that represents the fraction of the total number of non-null rows. To turn off dictionary encoding, set this fraction to a number that is less than the number of distinct keys in a dictionary. To always use dictionary encoding, set this threshold to `1`.
              * `enablePadding` (`bool`) - Set this to `true` to indicate that you want stripes to be padded to the HDFS block boundaries. This is useful if you intend to copy the data from Amazon S3 to HDFS before querying. The default is `false`.
              * `formatVersion` (`str`) - The version of the file to write. The possible values are `V0_11` and `V0_12`. The default is `V0_12`.
              * `paddingTolerance` (`float`) - A float between 0 and 1 that defines the tolerance for block padding as a decimal fraction of stripe size. The default value is `0.05`, which means 5 percent of stripe size. For the default values of 64 MiB ORC stripes and 256 MiB HDFS blocks, the default block padding tolerance of 5 percent reserves a maximum of 3.2 MiB for padding within the 256 MiB block. In such a case, if the available size within the block is more than 3.2 MiB, a new, smaller stripe is inserted to fit within that space. This ensures that no stripe crosses block boundaries and causes remote reads within a node-local task. Kinesis Data Firehose ignores this parameter when `enable_padding` is `false`.
              * `rowIndexStride` (`float`) - The number of rows between index entries. The default is `10000` and the minimum is `1000`.
              * `stripeSizeBytes` (`float`) - The number of bytes in each stripe. The default is 64 MiB and the minimum is 8 MiB.

            * `parquetSerDe` (`dict`) - Nested argument that specifies converting data to the Parquet format before storing it in Amazon S3. For more information, see [Apache Parquet](https://parquet.apache.org/documentation/latest/). More details below.
              * `blockSizeBytes` (`float`) - The Hadoop Distributed File System (HDFS) block size. This is useful if you intend to copy the data from Amazon S3 to HDFS before querying. The default is 256 MiB and the minimum is 64 MiB. Kinesis Data Firehose uses this value for padding calculations.
              * `compression` (`str`) - The compression code to use over data blocks. The possible values are `UNCOMPRESSED`, `SNAPPY`, and `GZIP`, with the default being `SNAPPY`. Use `SNAPPY` for higher decompression speed. Use `GZIP` if the compression ratio is more important than speed.
              * `enableDictionaryCompression` (`bool`) - Indicates whether to enable dictionary compression.
              * `maxPaddingBytes` (`float`) - The maximum amount of padding to apply. This is useful if you intend to copy the data from Amazon S3 to HDFS before querying. The default is `0`.
              * `pageSizeBytes` (`float`) - The Parquet page size. Column chunks are divided into pages. A page is conceptually an indivisible unit (in terms of compression and encoding). The minimum value is 64 KiB and the default is 1 MiB.
              * `writerVersion` (`str`) - Indicates the version of row format to output. The possible values are `V1` and `V2`. The default is `V1`.

        * `schemaConfiguration` (`dict`) - Nested argument that specifies the AWS Glue Data Catalog table that contains the column information. More details below.
          * `catalog_id` (`str`) - The ID of the AWS Glue Data Catalog. If you don't supply this, the AWS account ID is used by default.
          * `database_name` (`str`) - Specifies the name of the AWS Glue database that contains the schema for the output data.
          * `region` (`str`) - If you don't specify an AWS Region, the default is the current region.
          * `role_arn` (`str`) - The role that Kinesis Data Firehose can use to access AWS Glue. This role must be in the same account you use for Kinesis Data Firehose. Cross-account roles aren't allowed.
          * `table_name` (`str`) - Specifies the AWS Glue table that contains the column information that constitutes your data schema.
          * `version_id` (`str`) - Specifies the table version for the output data schema. Defaults to `LATEST`.

      * `errorOutputPrefix` (`str`) - Prefix added to failed records before writing them to S3. This prefix appears immediately following the bucket name.
      * `kms_key_arn` (`str`) - Specifies the KMS key ARN the stream will use to encrypt data. If not set, no encryption will
        be used.
      * `prefix` (`str`) - The "YYYY/MM/DD/HH" time format prefix is automatically used for delivered S3 files. You can specify an extra prefix to be added in front of the time format prefix. Note that if the prefix ends with a slash, it appears as a folder in the S3 bucket
      * `processingConfiguration` (`dict`) - The data processing configuration.  More details are given below.
        * `enabled` (`bool`) - Enables or disables data processing.
        * `processors` (`list`) - Array of data processors. More details are given below
          * `parameters` (`list`) - Array of processor parameters. More details are given below
            * `parameterName` (`str`) - Parameter name. Valid Values: `LambdaArn`, `NumberOfRetries`, `RoleArn`, `BufferSizeInMBs`, `BufferIntervalInSeconds`
            * `parameterValue` (`str`) - Parameter value. Must be between 1 and 512 length (inclusive). When providing a Lambda ARN, you should specify the resource version as well.

          * `type` (`str`) - The type of processor. Valid Values: `Lambda`

      * `role_arn` (`str`) - The role that Kinesis Data Firehose can use to access AWS Glue. This role must be in the same account you use for Kinesis Data Firehose. Cross-account roles aren't allowed.
      * `s3BackupConfiguration` (`dict`) - The configuration for backup in Amazon S3. Required if `s3_backup_mode` is `Enabled`. Supports the same fields as `s3_configuration` object.
        * `bucketArn` (`str`) - The ARN of the S3 bucket
        * `bufferInterval` (`float`) - Buffer incoming data for the specified period of time, in seconds, before delivering it to the destination. The default value is 300.
        * `bufferSize` (`float`) - Buffer incoming data to the specified size, in MBs, before delivering it to the destination. The default value is 5.
          We recommend setting SizeInMBs to a value greater than the amount of data you typically ingest into the delivery stream in 10 seconds. For example, if you typically ingest data at 1 MB/sec set SizeInMBs to be 10 MB or higher.
        * `cloudwatch_logging_options` (`dict`) - The CloudWatch Logging Options for the delivery stream. More details are given below
          * `enabled` (`bool`) - Enables or disables the logging. Defaults to `false`.
          * `log_group_name` (`str`) - The CloudWatch group name for logging. This value is required if `enabled` is true.
          * `logStreamName` (`str`) - The CloudWatch log stream name for logging. This value is required if `enabled` is true.

        * `compressionFormat` (`str`) - The compression format. If no value is specified, the default is UNCOMPRESSED. Other supported values are GZIP, ZIP & Snappy. If the destination is redshift you cannot use ZIP or Snappy.
        * `kms_key_arn` (`str`) - Specifies the KMS key ARN the stream will use to encrypt data. If not set, no encryption will
          be used.
        * `prefix` (`str`) - The "YYYY/MM/DD/HH" time format prefix is automatically used for delivered S3 files. You can specify an extra prefix to be added in front of the time format prefix. Note that if the prefix ends with a slash, it appears as a folder in the S3 bucket
        * `role_arn` (`str`) - The role that Kinesis Data Firehose can use to access AWS Glue. This role must be in the same account you use for Kinesis Data Firehose. Cross-account roles aren't allowed.

      * `s3BackupMode` (`str`) - The Amazon S3 backup mode.  Valid values are `Disabled` and `Enabled`.  Default value is `Disabled`.
    """
    kinesis_source_configuration: pulumi.Output[dict]
    """
    Allows the ability to specify the kinesis stream that is used as the source of the firehose delivery stream.

      * `kinesisStreamArn` (`str`) - The kinesis stream used as the source of the firehose delivery stream.
      * `role_arn` (`str`) - The ARN of the role that provides access to the source Kinesis stream.
    """
    name: pulumi.Output[str]
    """
    A name to identify the stream. This is unique to the
    AWS account and region the Stream is created in.
    """
    redshift_configuration: pulumi.Output[dict]
    """
    Configuration options if redshift is the destination.
    Using `redshift_configuration` requires the user to also specify a
    `s3_configuration` block. More details are given below.

      * `cloudwatch_logging_options` (`dict`) - The CloudWatch Logging Options for the delivery stream. More details are given below
        * `enabled` (`bool`) - Enables or disables the logging. Defaults to `false`.
        * `log_group_name` (`str`) - The CloudWatch group name for logging. This value is required if `enabled` is true.
        * `logStreamName` (`str`) - The CloudWatch log stream name for logging. This value is required if `enabled` is true.

      * `clusterJdbcurl` (`str`) - The jdbcurl of the redshift cluster.
      * `copyOptions` (`str`) - Copy options for copying the data from the s3 intermediate bucket into redshift, for example to change the default delimiter. For valid values, see the [AWS documentation](http://docs.aws.amazon.com/firehose/latest/APIReference/API_CopyCommand.html)
      * `dataTableColumns` (`str`) - The data table columns that will be targeted by the copy command.
      * `dataTableName` (`str`) - The name of the table in the redshift cluster that the s3 bucket will copy to.
      * `password` (`str`) - The password for the username above.
      * `processingConfiguration` (`dict`) - The data processing configuration.  More details are given below.
        * `enabled` (`bool`) - Enables or disables data processing.
        * `processors` (`list`) - Array of data processors. More details are given below
          * `parameters` (`list`) - Array of processor parameters. More details are given below
            * `parameterName` (`str`) - Parameter name. Valid Values: `LambdaArn`, `NumberOfRetries`, `RoleArn`, `BufferSizeInMBs`, `BufferIntervalInSeconds`
            * `parameterValue` (`str`) - Parameter value. Must be between 1 and 512 length (inclusive). When providing a Lambda ARN, you should specify the resource version as well.

          * `type` (`str`) - The type of processor. Valid Values: `Lambda`

      * `retryDuration` (`float`) - The length of time during which Firehose retries delivery after a failure, starting from the initial request and including the first attempt. The default value is 3600 seconds (60 minutes). Firehose does not retry if the value of DurationInSeconds is 0 (zero) or if the first delivery attempt takes longer than the current value.
      * `role_arn` (`str`) - The arn of the role the stream assumes.
      * `s3BackupConfiguration` (`dict`) - The configuration for backup in Amazon S3. Required if `s3_backup_mode` is `Enabled`. Supports the same fields as `s3_configuration` object.
        * `bucketArn` (`str`) - The ARN of the S3 bucket
        * `bufferInterval` (`float`) - Buffer incoming data for the specified period of time, in seconds, before delivering it to the destination. The default value is 300.
        * `bufferSize` (`float`) - Buffer incoming data to the specified size, in MBs, before delivering it to the destination. The default value is 5.
          We recommend setting SizeInMBs to a value greater than the amount of data you typically ingest into the delivery stream in 10 seconds. For example, if you typically ingest data at 1 MB/sec set SizeInMBs to be 10 MB or higher.
        * `cloudwatch_logging_options` (`dict`) - The CloudWatch Logging Options for the delivery stream. More details are given below
          * `enabled` (`bool`) - Enables or disables the logging. Defaults to `false`.
          * `log_group_name` (`str`) - The CloudWatch group name for logging. This value is required if `enabled` is true.
          * `logStreamName` (`str`) - The CloudWatch log stream name for logging. This value is required if `enabled` is true.

        * `compressionFormat` (`str`) - The compression format. If no value is specified, the default is UNCOMPRESSED. Other supported values are GZIP, ZIP & Snappy. If the destination is redshift you cannot use ZIP or Snappy.
        * `kms_key_arn` (`str`) - Specifies the KMS key ARN the stream will use to encrypt data. If not set, no encryption will
          be used.
        * `prefix` (`str`) - The "YYYY/MM/DD/HH" time format prefix is automatically used for delivered S3 files. You can specify an extra prefix to be added in front of the time format prefix. Note that if the prefix ends with a slash, it appears as a folder in the S3 bucket
        * `role_arn` (`str`) - The role that Kinesis Data Firehose can use to access AWS Glue. This role must be in the same account you use for Kinesis Data Firehose. Cross-account roles aren't allowed.

      * `s3BackupMode` (`str`) - The Amazon S3 backup mode.  Valid values are `Disabled` and `Enabled`.  Default value is `Disabled`.
      * `username` (`str`) - The username that the firehose delivery stream will assume. It is strongly recommended that the username and password provided is used exclusively for Amazon Kinesis Firehose purposes, and that the permissions for the account are restricted for Amazon Redshift INSERT permissions.
    """
    s3_configuration: pulumi.Output[dict]
    """
    Required for non-S3 destinations. For S3 destination, use `extended_s3_configuration` instead. Configuration options for the s3 destination (or the intermediate bucket if the destination
    is redshift). More details are given below.

      * `bucketArn` (`str`) - The ARN of the S3 bucket
      * `bufferInterval` (`float`) - Buffer incoming data for the specified period of time, in seconds, before delivering it to the destination. The default value is 300.
      * `bufferSize` (`float`) - Buffer incoming data to the specified size, in MBs, before delivering it to the destination. The default value is 5.
        We recommend setting SizeInMBs to a value greater than the amount of data you typically ingest into the delivery stream in 10 seconds. For example, if you typically ingest data at 1 MB/sec set SizeInMBs to be 10 MB or higher.
      * `cloudwatch_logging_options` (`dict`) - The CloudWatch Logging Options for the delivery stream. More details are given below
        * `enabled` (`bool`) - Enables or disables the logging. Defaults to `false`.
        * `log_group_name` (`str`) - The CloudWatch group name for logging. This value is required if `enabled` is true.
        * `logStreamName` (`str`) - The CloudWatch log stream name for logging. This value is required if `enabled` is true.

      * `compressionFormat` (`str`) - The compression format. If no value is specified, the default is UNCOMPRESSED. Other supported values are GZIP, ZIP & Snappy. If the destination is redshift you cannot use ZIP or Snappy.
      * `kms_key_arn` (`str`) - Specifies the KMS key ARN the stream will use to encrypt data. If not set, no encryption will
        be used.
      * `prefix` (`str`) - The "YYYY/MM/DD/HH" time format prefix is automatically used for delivered S3 files. You can specify an extra prefix to be added in front of the time format prefix. Note that if the prefix ends with a slash, it appears as a folder in the S3 bucket
      * `role_arn` (`str`) - The role that Kinesis Data Firehose can use to access AWS Glue. This role must be in the same account you use for Kinesis Data Firehose. Cross-account roles aren't allowed.
    """
    server_side_encryption: pulumi.Output[dict]
    """
    Encrypt at rest options.
    Server-side encryption should not be enabled when a kinesis stream is configured as the source of the firehose delivery stream.

      * `enabled` (`bool`) - Whether to enable encryption at rest. Default is `false`.
    """
    splunk_configuration: pulumi.Output[dict]
    tags: pulumi.Output[dict]
    """
    A map of tags to assign to the resource.
    """
    version_id: pulumi.Output[str]
    """
    Specifies the table version for the output data schema. Defaults to `LATEST`.
    """
    def __init__(__self__, resource_name, opts=None, arn=None, destination=None, destination_id=None, elasticsearch_configuration=None, extended_s3_configuration=None, kinesis_source_configuration=None, name=None, redshift_configuration=None, s3_configuration=None, server_side_encryption=None, splunk_configuration=None, tags=None, version_id=None, __props__=None, __name__=None, __opts__=None):
        """
        Provides a Kinesis Firehose Delivery Stream resource. Amazon Kinesis Firehose is a fully managed, elastic service to easily deliver real-time data streams to destinations such as Amazon S3 and Amazon Redshift.

        For more details, see the [Amazon Kinesis Firehose Documentation](https://aws.amazon.com/documentation/firehose/).



        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] arn: The Amazon Resource Name (ARN) specifying the Stream
        :param pulumi.Input[str] destination: This is the destination to where the data is delivered. The only options are `s3` (Deprecated, use `extended_s3` instead), `extended_s3`, `redshift`, `elasticsearch`, and `splunk`.
        :param pulumi.Input[dict] elasticsearch_configuration: Configuration options if elasticsearch is the destination. More details are given below.
        :param pulumi.Input[dict] extended_s3_configuration: Enhanced configuration options for the s3 destination. More details are given below.
        :param pulumi.Input[dict] kinesis_source_configuration: Allows the ability to specify the kinesis stream that is used as the source of the firehose delivery stream.
        :param pulumi.Input[str] name: A name to identify the stream. This is unique to the
               AWS account and region the Stream is created in.
        :param pulumi.Input[dict] redshift_configuration: Configuration options if redshift is the destination.
               Using `redshift_configuration` requires the user to also specify a
               `s3_configuration` block. More details are given below.
        :param pulumi.Input[dict] s3_configuration: Required for non-S3 destinations. For S3 destination, use `extended_s3_configuration` instead. Configuration options for the s3 destination (or the intermediate bucket if the destination
               is redshift). More details are given below.
        :param pulumi.Input[dict] server_side_encryption: Encrypt at rest options.
               Server-side encryption should not be enabled when a kinesis stream is configured as the source of the firehose delivery stream.
        :param pulumi.Input[dict] tags: A map of tags to assign to the resource.
        :param pulumi.Input[str] version_id: Specifies the table version for the output data schema. Defaults to `LATEST`.

        The **elasticsearch_configuration** object supports the following:

          * `bufferingInterval` (`pulumi.Input[float]`) - Buffer incoming data for the specified period of time, in seconds between 60 to 900, before delivering it to the destination.  The default value is 300s.
          * `bufferingSize` (`pulumi.Input[float]`) - Buffer incoming data to the specified size, in MBs between 1 to 100, before delivering it to the destination.  The default value is 5MB.
          * `cloudwatch_logging_options` (`pulumi.Input[dict]`) - The CloudWatch Logging Options for the delivery stream. More details are given below
            * `enabled` (`pulumi.Input[bool]`) - Enables or disables the logging. Defaults to `false`.
            * `log_group_name` (`pulumi.Input[str]`) - The CloudWatch group name for logging. This value is required if `enabled` is true.
            * `logStreamName` (`pulumi.Input[str]`) - The CloudWatch log stream name for logging. This value is required if `enabled` is true.

          * `domainArn` (`pulumi.Input[str]`) - The ARN of the Amazon ES domain.  The IAM role must have permission for `DescribeElasticsearchDomain`, `DescribeElasticsearchDomains`, and `DescribeElasticsearchDomainConfig` after assuming `RoleARN`.  The pattern needs to be `arn:.*`.
          * `indexName` (`pulumi.Input[str]`) - The Elasticsearch index name.
          * `indexRotationPeriod` (`pulumi.Input[str]`) - The Elasticsearch index rotation period.  Index rotation appends a timestamp to the IndexName to facilitate expiration of old data.  Valid values are `NoRotation`, `OneHour`, `OneDay`, `OneWeek`, and `OneMonth`.  The default value is `OneDay`.
          * `processingConfiguration` (`pulumi.Input[dict]`) - The data processing configuration.  More details are given below.
            * `enabled` (`pulumi.Input[bool]`) - Enables or disables data processing.
            * `processors` (`pulumi.Input[list]`) - Array of data processors. More details are given below
              * `parameters` (`pulumi.Input[list]`) - Array of processor parameters. More details are given below
                * `parameterName` (`pulumi.Input[str]`) - Parameter name. Valid Values: `LambdaArn`, `NumberOfRetries`, `RoleArn`, `BufferSizeInMBs`, `BufferIntervalInSeconds`
                * `parameterValue` (`pulumi.Input[str]`) - Parameter value. Must be between 1 and 512 length (inclusive). When providing a Lambda ARN, you should specify the resource version as well.

              * `type` (`pulumi.Input[str]`) - The type of processor. Valid Values: `Lambda`

          * `retryDuration` (`pulumi.Input[float]`) - After an initial failure to deliver to Amazon Elasticsearch, the total amount of time, in seconds between 0 to 7200, during which Firehose re-attempts delivery (including the first attempt).  After this time has elapsed, the failed documents are written to Amazon S3.  The default value is 300s.  There will be no retry if the value is 0.
          * `role_arn` (`pulumi.Input[str]`) - The ARN of the IAM role to be assumed by Firehose for calling the Amazon ES Configuration API and for indexing documents.  The pattern needs to be `arn:.*`.
          * `s3BackupMode` (`pulumi.Input[str]`) - Defines how documents should be delivered to Amazon S3.  Valid values are `FailedDocumentsOnly` and `AllDocuments`.  Default value is `FailedDocumentsOnly`.
          * `typeName` (`pulumi.Input[str]`) - The Elasticsearch type name with maximum length of 100 characters.

        The **extended_s3_configuration** object supports the following:

          * `bucketArn` (`pulumi.Input[str]`) - The ARN of the S3 bucket
          * `bufferInterval` (`pulumi.Input[float]`) - Buffer incoming data for the specified period of time, in seconds, before delivering it to the destination. The default value is 300.
          * `bufferSize` (`pulumi.Input[float]`) - Buffer incoming data to the specified size, in MBs, before delivering it to the destination. The default value is 5.
            We recommend setting SizeInMBs to a value greater than the amount of data you typically ingest into the delivery stream in 10 seconds. For example, if you typically ingest data at 1 MB/sec set SizeInMBs to be 10 MB or higher.
          * `cloudwatch_logging_options` (`pulumi.Input[dict]`) - The CloudWatch Logging Options for the delivery stream. More details are given below
            * `enabled` (`pulumi.Input[bool]`) - Enables or disables the logging. Defaults to `false`.
            * `log_group_name` (`pulumi.Input[str]`) - The CloudWatch group name for logging. This value is required if `enabled` is true.
            * `logStreamName` (`pulumi.Input[str]`) - The CloudWatch log stream name for logging. This value is required if `enabled` is true.

          * `compressionFormat` (`pulumi.Input[str]`) - The compression format. If no value is specified, the default is UNCOMPRESSED. Other supported values are GZIP, ZIP & Snappy. If the destination is redshift you cannot use ZIP or Snappy.
          * `dataFormatConversionConfiguration` (`pulumi.Input[dict]`) - Nested argument for the serializer, deserializer, and schema for converting data from the JSON format to the Parquet or ORC format before writing it to Amazon S3. More details given below.
            * `enabled` (`pulumi.Input[bool]`) - Defaults to `true`. Set it to `false` if you want to disable format conversion while preserving the configuration details.
            * `inputFormatConfiguration` (`pulumi.Input[dict]`) - Nested argument that specifies the deserializer that you want Kinesis Data Firehose to use to convert the format of your data from JSON. More details below.
              * `deserializer` (`pulumi.Input[dict]`) - Nested argument that specifies which deserializer to use. You can choose either the Apache Hive JSON SerDe or the OpenX JSON SerDe. More details below.
                * `hiveJsonSerDe` (`pulumi.Input[dict]`) - Nested argument that specifies the native Hive / HCatalog JsonSerDe. More details below.
                  * `timestampFormats` (`pulumi.Input[list]`) - A list of how you want Kinesis Data Firehose to parse the date and time stamps that may be present in your input data JSON. To specify these format strings, follow the pattern syntax of JodaTime's DateTimeFormat format strings. For more information, see [Class DateTimeFormat](https://www.joda.org/joda-time/apidocs/org/joda/time/format/DateTimeFormat.html). You can also use the special value millis to parse time stamps in epoch milliseconds. If you don't specify a format, Kinesis Data Firehose uses java.sql.Timestamp::valueOf by default.

                * `openXJsonSerDe` (`pulumi.Input[dict]`) - Nested argument that specifies the OpenX SerDe. More details below.
                  * `caseInsensitive` (`pulumi.Input[bool]`) - When set to true, which is the default, Kinesis Data Firehose converts JSON keys to lowercase before deserializing them.
                  * `columnToJsonKeyMappings` (`pulumi.Input[dict]`) - A map of column names to JSON keys that aren't identical to the column names. This is useful when the JSON contains keys that are Hive keywords. For example, timestamp is a Hive keyword. If you have a JSON key named timestamp, set this parameter to `{ ts = "timestamp" }` to map this key to a column named ts.
                  * `convertDotsInJsonKeysToUnderscores` (`pulumi.Input[bool]`) - When set to `true`, specifies that the names of the keys include dots and that you want Kinesis Data Firehose to replace them with underscores. This is useful because Apache Hive does not allow dots in column names. For example, if the JSON contains a key whose name is "a.b", you can define the column name to be "a_b" when using this option. Defaults to `false`.

            * `outputFormatConfiguration` (`pulumi.Input[dict]`) - Nested argument that specifies the serializer that you want Kinesis Data Firehose to use to convert the format of your data to the Parquet or ORC format. More details below.
              * `serializer` (`pulumi.Input[dict]`) - Nested argument that specifies which serializer to use. You can choose either the ORC SerDe or the Parquet SerDe. More details below.
                * `orcSerDe` (`pulumi.Input[dict]`) - Nested argument that specifies converting data to the ORC format before storing it in Amazon S3. For more information, see [Apache ORC](https://orc.apache.org/docs/). More details below.
                  * `blockSizeBytes` (`pulumi.Input[float]`) - The Hadoop Distributed File System (HDFS) block size. This is useful if you intend to copy the data from Amazon S3 to HDFS before querying. The default is 256 MiB and the minimum is 64 MiB. Kinesis Data Firehose uses this value for padding calculations.
                  * `bloomFilterColumns` (`pulumi.Input[list]`) - A list of column names for which you want Kinesis Data Firehose to create bloom filters.
                  * `bloomFilterFalsePositiveProbability` (`pulumi.Input[float]`) - The Bloom filter false positive probability (FPP). The lower the FPP, the bigger the Bloom filter. The default value is `0.05`, the minimum is `0`, and the maximum is `1`.
                  * `compression` (`pulumi.Input[str]`) - The compression code to use over data blocks. The possible values are `UNCOMPRESSED`, `SNAPPY`, and `GZIP`, with the default being `SNAPPY`. Use `SNAPPY` for higher decompression speed. Use `GZIP` if the compression ratio is more important than speed.
                  * `dictionaryKeyThreshold` (`pulumi.Input[float]`) - A float that represents the fraction of the total number of non-null rows. To turn off dictionary encoding, set this fraction to a number that is less than the number of distinct keys in a dictionary. To always use dictionary encoding, set this threshold to `1`.
                  * `enablePadding` (`pulumi.Input[bool]`) - Set this to `true` to indicate that you want stripes to be padded to the HDFS block boundaries. This is useful if you intend to copy the data from Amazon S3 to HDFS before querying. The default is `false`.
                  * `formatVersion` (`pulumi.Input[str]`) - The version of the file to write. The possible values are `V0_11` and `V0_12`. The default is `V0_12`.
                  * `paddingTolerance` (`pulumi.Input[float]`) - A float between 0 and 1 that defines the tolerance for block padding as a decimal fraction of stripe size. The default value is `0.05`, which means 5 percent of stripe size. For the default values of 64 MiB ORC stripes and 256 MiB HDFS blocks, the default block padding tolerance of 5 percent reserves a maximum of 3.2 MiB for padding within the 256 MiB block. In such a case, if the available size within the block is more than 3.2 MiB, a new, smaller stripe is inserted to fit within that space. This ensures that no stripe crosses block boundaries and causes remote reads within a node-local task. Kinesis Data Firehose ignores this parameter when `enable_padding` is `false`.
                  * `rowIndexStride` (`pulumi.Input[float]`) - The number of rows between index entries. The default is `10000` and the minimum is `1000`.
                  * `stripeSizeBytes` (`pulumi.Input[float]`) - The number of bytes in each stripe. The default is 64 MiB and the minimum is 8 MiB.

                * `parquetSerDe` (`pulumi.Input[dict]`) - Nested argument that specifies converting data to the Parquet format before storing it in Amazon S3. For more information, see [Apache Parquet](https://parquet.apache.org/documentation/latest/). More details below.
                  * `blockSizeBytes` (`pulumi.Input[float]`) - The Hadoop Distributed File System (HDFS) block size. This is useful if you intend to copy the data from Amazon S3 to HDFS before querying. The default is 256 MiB and the minimum is 64 MiB. Kinesis Data Firehose uses this value for padding calculations.
                  * `compression` (`pulumi.Input[str]`) - The compression code to use over data blocks. The possible values are `UNCOMPRESSED`, `SNAPPY`, and `GZIP`, with the default being `SNAPPY`. Use `SNAPPY` for higher decompression speed. Use `GZIP` if the compression ratio is more important than speed.
                  * `enableDictionaryCompression` (`pulumi.Input[bool]`) - Indicates whether to enable dictionary compression.
                  * `maxPaddingBytes` (`pulumi.Input[float]`) - The maximum amount of padding to apply. This is useful if you intend to copy the data from Amazon S3 to HDFS before querying. The default is `0`.
                  * `pageSizeBytes` (`pulumi.Input[float]`) - The Parquet page size. Column chunks are divided into pages. A page is conceptually an indivisible unit (in terms of compression and encoding). The minimum value is 64 KiB and the default is 1 MiB.
                  * `writerVersion` (`pulumi.Input[str]`) - Indicates the version of row format to output. The possible values are `V1` and `V2`. The default is `V1`.

            * `schemaConfiguration` (`pulumi.Input[dict]`) - Nested argument that specifies the AWS Glue Data Catalog table that contains the column information. More details below.
              * `catalog_id` (`pulumi.Input[str]`) - The ID of the AWS Glue Data Catalog. If you don't supply this, the AWS account ID is used by default.
              * `database_name` (`pulumi.Input[str]`) - Specifies the name of the AWS Glue database that contains the schema for the output data.
              * `region` (`pulumi.Input[str]`) - If you don't specify an AWS Region, the default is the current region.
              * `role_arn` (`pulumi.Input[str]`) - The role that Kinesis Data Firehose can use to access AWS Glue. This role must be in the same account you use for Kinesis Data Firehose. Cross-account roles aren't allowed.
              * `table_name` (`pulumi.Input[str]`) - Specifies the AWS Glue table that contains the column information that constitutes your data schema.
              * `version_id` (`pulumi.Input[str]`) - Specifies the table version for the output data schema. Defaults to `LATEST`.

          * `errorOutputPrefix` (`pulumi.Input[str]`) - Prefix added to failed records before writing them to S3. This prefix appears immediately following the bucket name.
          * `kms_key_arn` (`pulumi.Input[str]`) - Specifies the KMS key ARN the stream will use to encrypt data. If not set, no encryption will
            be used.
          * `prefix` (`pulumi.Input[str]`) - The "YYYY/MM/DD/HH" time format prefix is automatically used for delivered S3 files. You can specify an extra prefix to be added in front of the time format prefix. Note that if the prefix ends with a slash, it appears as a folder in the S3 bucket
          * `processingConfiguration` (`pulumi.Input[dict]`) - The data processing configuration.  More details are given below.
            * `enabled` (`pulumi.Input[bool]`) - Enables or disables data processing.
            * `processors` (`pulumi.Input[list]`) - Array of data processors. More details are given below
              * `parameters` (`pulumi.Input[list]`) - Array of processor parameters. More details are given below
                * `parameterName` (`pulumi.Input[str]`) - Parameter name. Valid Values: `LambdaArn`, `NumberOfRetries`, `RoleArn`, `BufferSizeInMBs`, `BufferIntervalInSeconds`
                * `parameterValue` (`pulumi.Input[str]`) - Parameter value. Must be between 1 and 512 length (inclusive). When providing a Lambda ARN, you should specify the resource version as well.

              * `type` (`pulumi.Input[str]`) - The type of processor. Valid Values: `Lambda`

          * `role_arn` (`pulumi.Input[str]`) - The role that Kinesis Data Firehose can use to access AWS Glue. This role must be in the same account you use for Kinesis Data Firehose. Cross-account roles aren't allowed.
          * `s3BackupConfiguration` (`pulumi.Input[dict]`) - The configuration for backup in Amazon S3. Required if `s3_backup_mode` is `Enabled`. Supports the same fields as `s3_configuration` object.
            * `bucketArn` (`pulumi.Input[str]`) - The ARN of the S3 bucket
            * `bufferInterval` (`pulumi.Input[float]`) - Buffer incoming data for the specified period of time, in seconds, before delivering it to the destination. The default value is 300.
            * `bufferSize` (`pulumi.Input[float]`) - Buffer incoming data to the specified size, in MBs, before delivering it to the destination. The default value is 5.
              We recommend setting SizeInMBs to a value greater than the amount of data you typically ingest into the delivery stream in 10 seconds. For example, if you typically ingest data at 1 MB/sec set SizeInMBs to be 10 MB or higher.
            * `cloudwatch_logging_options` (`pulumi.Input[dict]`) - The CloudWatch Logging Options for the delivery stream. More details are given below
              * `enabled` (`pulumi.Input[bool]`) - Enables or disables the logging. Defaults to `false`.
              * `log_group_name` (`pulumi.Input[str]`) - The CloudWatch group name for logging. This value is required if `enabled` is true.
              * `logStreamName` (`pulumi.Input[str]`) - The CloudWatch log stream name for logging. This value is required if `enabled` is true.

            * `compressionFormat` (`pulumi.Input[str]`) - The compression format. If no value is specified, the default is UNCOMPRESSED. Other supported values are GZIP, ZIP & Snappy. If the destination is redshift you cannot use ZIP or Snappy.
            * `kms_key_arn` (`pulumi.Input[str]`) - Specifies the KMS key ARN the stream will use to encrypt data. If not set, no encryption will
              be used.
            * `prefix` (`pulumi.Input[str]`) - The "YYYY/MM/DD/HH" time format prefix is automatically used for delivered S3 files. You can specify an extra prefix to be added in front of the time format prefix. Note that if the prefix ends with a slash, it appears as a folder in the S3 bucket
            * `role_arn` (`pulumi.Input[str]`) - The role that Kinesis Data Firehose can use to access AWS Glue. This role must be in the same account you use for Kinesis Data Firehose. Cross-account roles aren't allowed.

          * `s3BackupMode` (`pulumi.Input[str]`) - The Amazon S3 backup mode.  Valid values are `Disabled` and `Enabled`.  Default value is `Disabled`.

        The **kinesis_source_configuration** object supports the following:

          * `kinesisStreamArn` (`pulumi.Input[str]`) - The kinesis stream used as the source of the firehose delivery stream.
          * `role_arn` (`pulumi.Input[str]`) - The ARN of the role that provides access to the source Kinesis stream.

        The **redshift_configuration** object supports the following:

          * `cloudwatch_logging_options` (`pulumi.Input[dict]`) - The CloudWatch Logging Options for the delivery stream. More details are given below
            * `enabled` (`pulumi.Input[bool]`) - Enables or disables the logging. Defaults to `false`.
            * `log_group_name` (`pulumi.Input[str]`) - The CloudWatch group name for logging. This value is required if `enabled` is true.
            * `logStreamName` (`pulumi.Input[str]`) - The CloudWatch log stream name for logging. This value is required if `enabled` is true.

          * `clusterJdbcurl` (`pulumi.Input[str]`) - The jdbcurl of the redshift cluster.
          * `copyOptions` (`pulumi.Input[str]`) - Copy options for copying the data from the s3 intermediate bucket into redshift, for example to change the default delimiter. For valid values, see the [AWS documentation](http://docs.aws.amazon.com/firehose/latest/APIReference/API_CopyCommand.html)
          * `dataTableColumns` (`pulumi.Input[str]`) - The data table columns that will be targeted by the copy command.
          * `dataTableName` (`pulumi.Input[str]`) - The name of the table in the redshift cluster that the s3 bucket will copy to.
          * `password` (`pulumi.Input[str]`) - The password for the username above.
          * `processingConfiguration` (`pulumi.Input[dict]`) - The data processing configuration.  More details are given below.
            * `enabled` (`pulumi.Input[bool]`) - Enables or disables data processing.
            * `processors` (`pulumi.Input[list]`) - Array of data processors. More details are given below
              * `parameters` (`pulumi.Input[list]`) - Array of processor parameters. More details are given below
                * `parameterName` (`pulumi.Input[str]`) - Parameter name. Valid Values: `LambdaArn`, `NumberOfRetries`, `RoleArn`, `BufferSizeInMBs`, `BufferIntervalInSeconds`
                * `parameterValue` (`pulumi.Input[str]`) - Parameter value. Must be between 1 and 512 length (inclusive). When providing a Lambda ARN, you should specify the resource version as well.

              * `type` (`pulumi.Input[str]`) - The type of processor. Valid Values: `Lambda`

          * `retryDuration` (`pulumi.Input[float]`) - The length of time during which Firehose retries delivery after a failure, starting from the initial request and including the first attempt. The default value is 3600 seconds (60 minutes). Firehose does not retry if the value of DurationInSeconds is 0 (zero) or if the first delivery attempt takes longer than the current value.
          * `role_arn` (`pulumi.Input[str]`) - The arn of the role the stream assumes.
          * `s3BackupConfiguration` (`pulumi.Input[dict]`) - The configuration for backup in Amazon S3. Required if `s3_backup_mode` is `Enabled`. Supports the same fields as `s3_configuration` object.
            * `bucketArn` (`pulumi.Input[str]`) - The ARN of the S3 bucket
            * `bufferInterval` (`pulumi.Input[float]`) - Buffer incoming data for the specified period of time, in seconds, before delivering it to the destination. The default value is 300.
            * `bufferSize` (`pulumi.Input[float]`) - Buffer incoming data to the specified size, in MBs, before delivering it to the destination. The default value is 5.
              We recommend setting SizeInMBs to a value greater than the amount of data you typically ingest into the delivery stream in 10 seconds. For example, if you typically ingest data at 1 MB/sec set SizeInMBs to be 10 MB or higher.
            * `cloudwatch_logging_options` (`pulumi.Input[dict]`) - The CloudWatch Logging Options for the delivery stream. More details are given below
              * `enabled` (`pulumi.Input[bool]`) - Enables or disables the logging. Defaults to `false`.
              * `log_group_name` (`pulumi.Input[str]`) - The CloudWatch group name for logging. This value is required if `enabled` is true.
              * `logStreamName` (`pulumi.Input[str]`) - The CloudWatch log stream name for logging. This value is required if `enabled` is true.

            * `compressionFormat` (`pulumi.Input[str]`) - The compression format. If no value is specified, the default is UNCOMPRESSED. Other supported values are GZIP, ZIP & Snappy. If the destination is redshift you cannot use ZIP or Snappy.
            * `kms_key_arn` (`pulumi.Input[str]`) - Specifies the KMS key ARN the stream will use to encrypt data. If not set, no encryption will
              be used.
            * `prefix` (`pulumi.Input[str]`) - The "YYYY/MM/DD/HH" time format prefix is automatically used for delivered S3 files. You can specify an extra prefix to be added in front of the time format prefix. Note that if the prefix ends with a slash, it appears as a folder in the S3 bucket
            * `role_arn` (`pulumi.Input[str]`) - The role that Kinesis Data Firehose can use to access AWS Glue. This role must be in the same account you use for Kinesis Data Firehose. Cross-account roles aren't allowed.

          * `s3BackupMode` (`pulumi.Input[str]`) - The Amazon S3 backup mode.  Valid values are `Disabled` and `Enabled`.  Default value is `Disabled`.
          * `username` (`pulumi.Input[str]`) - The username that the firehose delivery stream will assume. It is strongly recommended that the username and password provided is used exclusively for Amazon Kinesis Firehose purposes, and that the permissions for the account are restricted for Amazon Redshift INSERT permissions.

        The **s3_configuration** object supports the following:

          * `bucketArn` (`pulumi.Input[str]`) - The ARN of the S3 bucket
          * `bufferInterval` (`pulumi.Input[float]`) - Buffer incoming data for the specified period of time, in seconds, before delivering it to the destination. The default value is 300.
          * `bufferSize` (`pulumi.Input[float]`) - Buffer incoming data to the specified size, in MBs, before delivering it to the destination. The default value is 5.
            We recommend setting SizeInMBs to a value greater than the amount of data you typically ingest into the delivery stream in 10 seconds. For example, if you typically ingest data at 1 MB/sec set SizeInMBs to be 10 MB or higher.
          * `cloudwatch_logging_options` (`pulumi.Input[dict]`) - The CloudWatch Logging Options for the delivery stream. More details are given below
            * `enabled` (`pulumi.Input[bool]`) - Enables or disables the logging. Defaults to `false`.
            * `log_group_name` (`pulumi.Input[str]`) - The CloudWatch group name for logging. This value is required if `enabled` is true.
            * `logStreamName` (`pulumi.Input[str]`) - The CloudWatch log stream name for logging. This value is required if `enabled` is true.

          * `compressionFormat` (`pulumi.Input[str]`) - The compression format. If no value is specified, the default is UNCOMPRESSED. Other supported values are GZIP, ZIP & Snappy. If the destination is redshift you cannot use ZIP or Snappy.
          * `kms_key_arn` (`pulumi.Input[str]`) - Specifies the KMS key ARN the stream will use to encrypt data. If not set, no encryption will
            be used.
          * `prefix` (`pulumi.Input[str]`) - The "YYYY/MM/DD/HH" time format prefix is automatically used for delivered S3 files. You can specify an extra prefix to be added in front of the time format prefix. Note that if the prefix ends with a slash, it appears as a folder in the S3 bucket
          * `role_arn` (`pulumi.Input[str]`) - The role that Kinesis Data Firehose can use to access AWS Glue. This role must be in the same account you use for Kinesis Data Firehose. Cross-account roles aren't allowed.

        The **server_side_encryption** object supports the following:

          * `enabled` (`pulumi.Input[bool]`) - Whether to enable encryption at rest. Default is `false`.

        The **splunk_configuration** object supports the following:

          * `cloudwatch_logging_options` (`pulumi.Input[dict]`) - The CloudWatch Logging Options for the delivery stream. More details are given below.
            * `enabled` (`pulumi.Input[bool]`) - Enables or disables the logging. Defaults to `false`.
            * `log_group_name` (`pulumi.Input[str]`) - The CloudWatch group name for logging. This value is required if `enabled` is true.
            * `logStreamName` (`pulumi.Input[str]`) - The CloudWatch log stream name for logging. This value is required if `enabled` is true.

          * `hecAcknowledgmentTimeout` (`pulumi.Input[float]`) - The amount of time, in seconds between 180 and 600, that Kinesis Firehose waits to receive an acknowledgment from Splunk after it sends it data.
          * `hecEndpoint` (`pulumi.Input[str]`) - The HTTP Event Collector (HEC) endpoint to which Kinesis Firehose sends your data.
          * `hecEndpointType` (`pulumi.Input[str]`) - The HEC endpoint type. Valid values are `Raw` or `Event`. The default value is `Raw`.
          * `hecToken` (`pulumi.Input[str]`) - The GUID that you obtain from your Splunk cluster when you create a new HEC endpoint.
          * `processingConfiguration` (`pulumi.Input[dict]`) - The data processing configuration.  More details are given below.
            * `enabled` (`pulumi.Input[bool]`) - Enables or disables data processing.
            * `processors` (`pulumi.Input[list]`) - Array of data processors. More details are given below
              * `parameters` (`pulumi.Input[list]`) - Array of processor parameters. More details are given below
                * `parameterName` (`pulumi.Input[str]`) - Parameter name. Valid Values: `LambdaArn`, `NumberOfRetries`, `RoleArn`, `BufferSizeInMBs`, `BufferIntervalInSeconds`
                * `parameterValue` (`pulumi.Input[str]`) - Parameter value. Must be between 1 and 512 length (inclusive). When providing a Lambda ARN, you should specify the resource version as well.

              * `type` (`pulumi.Input[str]`) - The type of processor. Valid Values: `Lambda`

          * `retryDuration` (`pulumi.Input[float]`) - After an initial failure to deliver to Splunk, the total amount of time, in seconds between 0 to 7200, during which Firehose re-attempts delivery (including the first attempt).  After this time has elapsed, the failed documents are written to Amazon S3.  The default value is 300s.  There will be no retry if the value is 0.
          * `s3BackupMode` (`pulumi.Input[str]`) - Defines how documents should be delivered to Amazon S3.  Valid values are `FailedEventsOnly` and `AllEvents`.  Default value is `FailedEventsOnly`.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['arn'] = arn
            if destination is None:
                raise TypeError("Missing required property 'destination'")
            __props__['destination'] = destination
            __props__['destination_id'] = destination_id
            __props__['elasticsearch_configuration'] = elasticsearch_configuration
            __props__['extended_s3_configuration'] = extended_s3_configuration
            __props__['kinesis_source_configuration'] = kinesis_source_configuration
            __props__['name'] = name
            __props__['redshift_configuration'] = redshift_configuration
            __props__['s3_configuration'] = s3_configuration
            __props__['server_side_encryption'] = server_side_encryption
            __props__['splunk_configuration'] = splunk_configuration
            __props__['tags'] = tags
            __props__['version_id'] = version_id
        super(FirehoseDeliveryStream, __self__).__init__(
            'aws:kinesis/firehoseDeliveryStream:FirehoseDeliveryStream',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, arn=None, destination=None, destination_id=None, elasticsearch_configuration=None, extended_s3_configuration=None, kinesis_source_configuration=None, name=None, redshift_configuration=None, s3_configuration=None, server_side_encryption=None, splunk_configuration=None, tags=None, version_id=None):
        """
        Get an existing FirehoseDeliveryStream resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] arn: The Amazon Resource Name (ARN) specifying the Stream
        :param pulumi.Input[str] destination: This is the destination to where the data is delivered. The only options are `s3` (Deprecated, use `extended_s3` instead), `extended_s3`, `redshift`, `elasticsearch`, and `splunk`.
        :param pulumi.Input[dict] elasticsearch_configuration: Configuration options if elasticsearch is the destination. More details are given below.
        :param pulumi.Input[dict] extended_s3_configuration: Enhanced configuration options for the s3 destination. More details are given below.
        :param pulumi.Input[dict] kinesis_source_configuration: Allows the ability to specify the kinesis stream that is used as the source of the firehose delivery stream.
        :param pulumi.Input[str] name: A name to identify the stream. This is unique to the
               AWS account and region the Stream is created in.
        :param pulumi.Input[dict] redshift_configuration: Configuration options if redshift is the destination.
               Using `redshift_configuration` requires the user to also specify a
               `s3_configuration` block. More details are given below.
        :param pulumi.Input[dict] s3_configuration: Required for non-S3 destinations. For S3 destination, use `extended_s3_configuration` instead. Configuration options for the s3 destination (or the intermediate bucket if the destination
               is redshift). More details are given below.
        :param pulumi.Input[dict] server_side_encryption: Encrypt at rest options.
               Server-side encryption should not be enabled when a kinesis stream is configured as the source of the firehose delivery stream.
        :param pulumi.Input[dict] tags: A map of tags to assign to the resource.
        :param pulumi.Input[str] version_id: Specifies the table version for the output data schema. Defaults to `LATEST`.

        The **elasticsearch_configuration** object supports the following:

          * `bufferingInterval` (`pulumi.Input[float]`) - Buffer incoming data for the specified period of time, in seconds between 60 to 900, before delivering it to the destination.  The default value is 300s.
          * `bufferingSize` (`pulumi.Input[float]`) - Buffer incoming data to the specified size, in MBs between 1 to 100, before delivering it to the destination.  The default value is 5MB.
          * `cloudwatch_logging_options` (`pulumi.Input[dict]`) - The CloudWatch Logging Options for the delivery stream. More details are given below
            * `enabled` (`pulumi.Input[bool]`) - Enables or disables the logging. Defaults to `false`.
            * `log_group_name` (`pulumi.Input[str]`) - The CloudWatch group name for logging. This value is required if `enabled` is true.
            * `logStreamName` (`pulumi.Input[str]`) - The CloudWatch log stream name for logging. This value is required if `enabled` is true.

          * `domainArn` (`pulumi.Input[str]`) - The ARN of the Amazon ES domain.  The IAM role must have permission for `DescribeElasticsearchDomain`, `DescribeElasticsearchDomains`, and `DescribeElasticsearchDomainConfig` after assuming `RoleARN`.  The pattern needs to be `arn:.*`.
          * `indexName` (`pulumi.Input[str]`) - The Elasticsearch index name.
          * `indexRotationPeriod` (`pulumi.Input[str]`) - The Elasticsearch index rotation period.  Index rotation appends a timestamp to the IndexName to facilitate expiration of old data.  Valid values are `NoRotation`, `OneHour`, `OneDay`, `OneWeek`, and `OneMonth`.  The default value is `OneDay`.
          * `processingConfiguration` (`pulumi.Input[dict]`) - The data processing configuration.  More details are given below.
            * `enabled` (`pulumi.Input[bool]`) - Enables or disables data processing.
            * `processors` (`pulumi.Input[list]`) - Array of data processors. More details are given below
              * `parameters` (`pulumi.Input[list]`) - Array of processor parameters. More details are given below
                * `parameterName` (`pulumi.Input[str]`) - Parameter name. Valid Values: `LambdaArn`, `NumberOfRetries`, `RoleArn`, `BufferSizeInMBs`, `BufferIntervalInSeconds`
                * `parameterValue` (`pulumi.Input[str]`) - Parameter value. Must be between 1 and 512 length (inclusive). When providing a Lambda ARN, you should specify the resource version as well.

              * `type` (`pulumi.Input[str]`) - The type of processor. Valid Values: `Lambda`

          * `retryDuration` (`pulumi.Input[float]`) - After an initial failure to deliver to Amazon Elasticsearch, the total amount of time, in seconds between 0 to 7200, during which Firehose re-attempts delivery (including the first attempt).  After this time has elapsed, the failed documents are written to Amazon S3.  The default value is 300s.  There will be no retry if the value is 0.
          * `role_arn` (`pulumi.Input[str]`) - The ARN of the IAM role to be assumed by Firehose for calling the Amazon ES Configuration API and for indexing documents.  The pattern needs to be `arn:.*`.
          * `s3BackupMode` (`pulumi.Input[str]`) - Defines how documents should be delivered to Amazon S3.  Valid values are `FailedDocumentsOnly` and `AllDocuments`.  Default value is `FailedDocumentsOnly`.
          * `typeName` (`pulumi.Input[str]`) - The Elasticsearch type name with maximum length of 100 characters.

        The **extended_s3_configuration** object supports the following:

          * `bucketArn` (`pulumi.Input[str]`) - The ARN of the S3 bucket
          * `bufferInterval` (`pulumi.Input[float]`) - Buffer incoming data for the specified period of time, in seconds, before delivering it to the destination. The default value is 300.
          * `bufferSize` (`pulumi.Input[float]`) - Buffer incoming data to the specified size, in MBs, before delivering it to the destination. The default value is 5.
            We recommend setting SizeInMBs to a value greater than the amount of data you typically ingest into the delivery stream in 10 seconds. For example, if you typically ingest data at 1 MB/sec set SizeInMBs to be 10 MB or higher.
          * `cloudwatch_logging_options` (`pulumi.Input[dict]`) - The CloudWatch Logging Options for the delivery stream. More details are given below
            * `enabled` (`pulumi.Input[bool]`) - Enables or disables the logging. Defaults to `false`.
            * `log_group_name` (`pulumi.Input[str]`) - The CloudWatch group name for logging. This value is required if `enabled` is true.
            * `logStreamName` (`pulumi.Input[str]`) - The CloudWatch log stream name for logging. This value is required if `enabled` is true.

          * `compressionFormat` (`pulumi.Input[str]`) - The compression format. If no value is specified, the default is UNCOMPRESSED. Other supported values are GZIP, ZIP & Snappy. If the destination is redshift you cannot use ZIP or Snappy.
          * `dataFormatConversionConfiguration` (`pulumi.Input[dict]`) - Nested argument for the serializer, deserializer, and schema for converting data from the JSON format to the Parquet or ORC format before writing it to Amazon S3. More details given below.
            * `enabled` (`pulumi.Input[bool]`) - Defaults to `true`. Set it to `false` if you want to disable format conversion while preserving the configuration details.
            * `inputFormatConfiguration` (`pulumi.Input[dict]`) - Nested argument that specifies the deserializer that you want Kinesis Data Firehose to use to convert the format of your data from JSON. More details below.
              * `deserializer` (`pulumi.Input[dict]`) - Nested argument that specifies which deserializer to use. You can choose either the Apache Hive JSON SerDe or the OpenX JSON SerDe. More details below.
                * `hiveJsonSerDe` (`pulumi.Input[dict]`) - Nested argument that specifies the native Hive / HCatalog JsonSerDe. More details below.
                  * `timestampFormats` (`pulumi.Input[list]`) - A list of how you want Kinesis Data Firehose to parse the date and time stamps that may be present in your input data JSON. To specify these format strings, follow the pattern syntax of JodaTime's DateTimeFormat format strings. For more information, see [Class DateTimeFormat](https://www.joda.org/joda-time/apidocs/org/joda/time/format/DateTimeFormat.html). You can also use the special value millis to parse time stamps in epoch milliseconds. If you don't specify a format, Kinesis Data Firehose uses java.sql.Timestamp::valueOf by default.

                * `openXJsonSerDe` (`pulumi.Input[dict]`) - Nested argument that specifies the OpenX SerDe. More details below.
                  * `caseInsensitive` (`pulumi.Input[bool]`) - When set to true, which is the default, Kinesis Data Firehose converts JSON keys to lowercase before deserializing them.
                  * `columnToJsonKeyMappings` (`pulumi.Input[dict]`) - A map of column names to JSON keys that aren't identical to the column names. This is useful when the JSON contains keys that are Hive keywords. For example, timestamp is a Hive keyword. If you have a JSON key named timestamp, set this parameter to `{ ts = "timestamp" }` to map this key to a column named ts.
                  * `convertDotsInJsonKeysToUnderscores` (`pulumi.Input[bool]`) - When set to `true`, specifies that the names of the keys include dots and that you want Kinesis Data Firehose to replace them with underscores. This is useful because Apache Hive does not allow dots in column names. For example, if the JSON contains a key whose name is "a.b", you can define the column name to be "a_b" when using this option. Defaults to `false`.

            * `outputFormatConfiguration` (`pulumi.Input[dict]`) - Nested argument that specifies the serializer that you want Kinesis Data Firehose to use to convert the format of your data to the Parquet or ORC format. More details below.
              * `serializer` (`pulumi.Input[dict]`) - Nested argument that specifies which serializer to use. You can choose either the ORC SerDe or the Parquet SerDe. More details below.
                * `orcSerDe` (`pulumi.Input[dict]`) - Nested argument that specifies converting data to the ORC format before storing it in Amazon S3. For more information, see [Apache ORC](https://orc.apache.org/docs/). More details below.
                  * `blockSizeBytes` (`pulumi.Input[float]`) - The Hadoop Distributed File System (HDFS) block size. This is useful if you intend to copy the data from Amazon S3 to HDFS before querying. The default is 256 MiB and the minimum is 64 MiB. Kinesis Data Firehose uses this value for padding calculations.
                  * `bloomFilterColumns` (`pulumi.Input[list]`) - A list of column names for which you want Kinesis Data Firehose to create bloom filters.
                  * `bloomFilterFalsePositiveProbability` (`pulumi.Input[float]`) - The Bloom filter false positive probability (FPP). The lower the FPP, the bigger the Bloom filter. The default value is `0.05`, the minimum is `0`, and the maximum is `1`.
                  * `compression` (`pulumi.Input[str]`) - The compression code to use over data blocks. The possible values are `UNCOMPRESSED`, `SNAPPY`, and `GZIP`, with the default being `SNAPPY`. Use `SNAPPY` for higher decompression speed. Use `GZIP` if the compression ratio is more important than speed.
                  * `dictionaryKeyThreshold` (`pulumi.Input[float]`) - A float that represents the fraction of the total number of non-null rows. To turn off dictionary encoding, set this fraction to a number that is less than the number of distinct keys in a dictionary. To always use dictionary encoding, set this threshold to `1`.
                  * `enablePadding` (`pulumi.Input[bool]`) - Set this to `true` to indicate that you want stripes to be padded to the HDFS block boundaries. This is useful if you intend to copy the data from Amazon S3 to HDFS before querying. The default is `false`.
                  * `formatVersion` (`pulumi.Input[str]`) - The version of the file to write. The possible values are `V0_11` and `V0_12`. The default is `V0_12`.
                  * `paddingTolerance` (`pulumi.Input[float]`) - A float between 0 and 1 that defines the tolerance for block padding as a decimal fraction of stripe size. The default value is `0.05`, which means 5 percent of stripe size. For the default values of 64 MiB ORC stripes and 256 MiB HDFS blocks, the default block padding tolerance of 5 percent reserves a maximum of 3.2 MiB for padding within the 256 MiB block. In such a case, if the available size within the block is more than 3.2 MiB, a new, smaller stripe is inserted to fit within that space. This ensures that no stripe crosses block boundaries and causes remote reads within a node-local task. Kinesis Data Firehose ignores this parameter when `enable_padding` is `false`.
                  * `rowIndexStride` (`pulumi.Input[float]`) - The number of rows between index entries. The default is `10000` and the minimum is `1000`.
                  * `stripeSizeBytes` (`pulumi.Input[float]`) - The number of bytes in each stripe. The default is 64 MiB and the minimum is 8 MiB.

                * `parquetSerDe` (`pulumi.Input[dict]`) - Nested argument that specifies converting data to the Parquet format before storing it in Amazon S3. For more information, see [Apache Parquet](https://parquet.apache.org/documentation/latest/). More details below.
                  * `blockSizeBytes` (`pulumi.Input[float]`) - The Hadoop Distributed File System (HDFS) block size. This is useful if you intend to copy the data from Amazon S3 to HDFS before querying. The default is 256 MiB and the minimum is 64 MiB. Kinesis Data Firehose uses this value for padding calculations.
                  * `compression` (`pulumi.Input[str]`) - The compression code to use over data blocks. The possible values are `UNCOMPRESSED`, `SNAPPY`, and `GZIP`, with the default being `SNAPPY`. Use `SNAPPY` for higher decompression speed. Use `GZIP` if the compression ratio is more important than speed.
                  * `enableDictionaryCompression` (`pulumi.Input[bool]`) - Indicates whether to enable dictionary compression.
                  * `maxPaddingBytes` (`pulumi.Input[float]`) - The maximum amount of padding to apply. This is useful if you intend to copy the data from Amazon S3 to HDFS before querying. The default is `0`.
                  * `pageSizeBytes` (`pulumi.Input[float]`) - The Parquet page size. Column chunks are divided into pages. A page is conceptually an indivisible unit (in terms of compression and encoding). The minimum value is 64 KiB and the default is 1 MiB.
                  * `writerVersion` (`pulumi.Input[str]`) - Indicates the version of row format to output. The possible values are `V1` and `V2`. The default is `V1`.

            * `schemaConfiguration` (`pulumi.Input[dict]`) - Nested argument that specifies the AWS Glue Data Catalog table that contains the column information. More details below.
              * `catalog_id` (`pulumi.Input[str]`) - The ID of the AWS Glue Data Catalog. If you don't supply this, the AWS account ID is used by default.
              * `database_name` (`pulumi.Input[str]`) - Specifies the name of the AWS Glue database that contains the schema for the output data.
              * `region` (`pulumi.Input[str]`) - If you don't specify an AWS Region, the default is the current region.
              * `role_arn` (`pulumi.Input[str]`) - The role that Kinesis Data Firehose can use to access AWS Glue. This role must be in the same account you use for Kinesis Data Firehose. Cross-account roles aren't allowed.
              * `table_name` (`pulumi.Input[str]`) - Specifies the AWS Glue table that contains the column information that constitutes your data schema.
              * `version_id` (`pulumi.Input[str]`) - Specifies the table version for the output data schema. Defaults to `LATEST`.

          * `errorOutputPrefix` (`pulumi.Input[str]`) - Prefix added to failed records before writing them to S3. This prefix appears immediately following the bucket name.
          * `kms_key_arn` (`pulumi.Input[str]`) - Specifies the KMS key ARN the stream will use to encrypt data. If not set, no encryption will
            be used.
          * `prefix` (`pulumi.Input[str]`) - The "YYYY/MM/DD/HH" time format prefix is automatically used for delivered S3 files. You can specify an extra prefix to be added in front of the time format prefix. Note that if the prefix ends with a slash, it appears as a folder in the S3 bucket
          * `processingConfiguration` (`pulumi.Input[dict]`) - The data processing configuration.  More details are given below.
            * `enabled` (`pulumi.Input[bool]`) - Enables or disables data processing.
            * `processors` (`pulumi.Input[list]`) - Array of data processors. More details are given below
              * `parameters` (`pulumi.Input[list]`) - Array of processor parameters. More details are given below
                * `parameterName` (`pulumi.Input[str]`) - Parameter name. Valid Values: `LambdaArn`, `NumberOfRetries`, `RoleArn`, `BufferSizeInMBs`, `BufferIntervalInSeconds`
                * `parameterValue` (`pulumi.Input[str]`) - Parameter value. Must be between 1 and 512 length (inclusive). When providing a Lambda ARN, you should specify the resource version as well.

              * `type` (`pulumi.Input[str]`) - The type of processor. Valid Values: `Lambda`

          * `role_arn` (`pulumi.Input[str]`) - The role that Kinesis Data Firehose can use to access AWS Glue. This role must be in the same account you use for Kinesis Data Firehose. Cross-account roles aren't allowed.
          * `s3BackupConfiguration` (`pulumi.Input[dict]`) - The configuration for backup in Amazon S3. Required if `s3_backup_mode` is `Enabled`. Supports the same fields as `s3_configuration` object.
            * `bucketArn` (`pulumi.Input[str]`) - The ARN of the S3 bucket
            * `bufferInterval` (`pulumi.Input[float]`) - Buffer incoming data for the specified period of time, in seconds, before delivering it to the destination. The default value is 300.
            * `bufferSize` (`pulumi.Input[float]`) - Buffer incoming data to the specified size, in MBs, before delivering it to the destination. The default value is 5.
              We recommend setting SizeInMBs to a value greater than the amount of data you typically ingest into the delivery stream in 10 seconds. For example, if you typically ingest data at 1 MB/sec set SizeInMBs to be 10 MB or higher.
            * `cloudwatch_logging_options` (`pulumi.Input[dict]`) - The CloudWatch Logging Options for the delivery stream. More details are given below
              * `enabled` (`pulumi.Input[bool]`) - Enables or disables the logging. Defaults to `false`.
              * `log_group_name` (`pulumi.Input[str]`) - The CloudWatch group name for logging. This value is required if `enabled` is true.
              * `logStreamName` (`pulumi.Input[str]`) - The CloudWatch log stream name for logging. This value is required if `enabled` is true.

            * `compressionFormat` (`pulumi.Input[str]`) - The compression format. If no value is specified, the default is UNCOMPRESSED. Other supported values are GZIP, ZIP & Snappy. If the destination is redshift you cannot use ZIP or Snappy.
            * `kms_key_arn` (`pulumi.Input[str]`) - Specifies the KMS key ARN the stream will use to encrypt data. If not set, no encryption will
              be used.
            * `prefix` (`pulumi.Input[str]`) - The "YYYY/MM/DD/HH" time format prefix is automatically used for delivered S3 files. You can specify an extra prefix to be added in front of the time format prefix. Note that if the prefix ends with a slash, it appears as a folder in the S3 bucket
            * `role_arn` (`pulumi.Input[str]`) - The role that Kinesis Data Firehose can use to access AWS Glue. This role must be in the same account you use for Kinesis Data Firehose. Cross-account roles aren't allowed.

          * `s3BackupMode` (`pulumi.Input[str]`) - The Amazon S3 backup mode.  Valid values are `Disabled` and `Enabled`.  Default value is `Disabled`.

        The **kinesis_source_configuration** object supports the following:

          * `kinesisStreamArn` (`pulumi.Input[str]`) - The kinesis stream used as the source of the firehose delivery stream.
          * `role_arn` (`pulumi.Input[str]`) - The ARN of the role that provides access to the source Kinesis stream.

        The **redshift_configuration** object supports the following:

          * `cloudwatch_logging_options` (`pulumi.Input[dict]`) - The CloudWatch Logging Options for the delivery stream. More details are given below
            * `enabled` (`pulumi.Input[bool]`) - Enables or disables the logging. Defaults to `false`.
            * `log_group_name` (`pulumi.Input[str]`) - The CloudWatch group name for logging. This value is required if `enabled` is true.
            * `logStreamName` (`pulumi.Input[str]`) - The CloudWatch log stream name for logging. This value is required if `enabled` is true.

          * `clusterJdbcurl` (`pulumi.Input[str]`) - The jdbcurl of the redshift cluster.
          * `copyOptions` (`pulumi.Input[str]`) - Copy options for copying the data from the s3 intermediate bucket into redshift, for example to change the default delimiter. For valid values, see the [AWS documentation](http://docs.aws.amazon.com/firehose/latest/APIReference/API_CopyCommand.html)
          * `dataTableColumns` (`pulumi.Input[str]`) - The data table columns that will be targeted by the copy command.
          * `dataTableName` (`pulumi.Input[str]`) - The name of the table in the redshift cluster that the s3 bucket will copy to.
          * `password` (`pulumi.Input[str]`) - The password for the username above.
          * `processingConfiguration` (`pulumi.Input[dict]`) - The data processing configuration.  More details are given below.
            * `enabled` (`pulumi.Input[bool]`) - Enables or disables data processing.
            * `processors` (`pulumi.Input[list]`) - Array of data processors. More details are given below
              * `parameters` (`pulumi.Input[list]`) - Array of processor parameters. More details are given below
                * `parameterName` (`pulumi.Input[str]`) - Parameter name. Valid Values: `LambdaArn`, `NumberOfRetries`, `RoleArn`, `BufferSizeInMBs`, `BufferIntervalInSeconds`
                * `parameterValue` (`pulumi.Input[str]`) - Parameter value. Must be between 1 and 512 length (inclusive). When providing a Lambda ARN, you should specify the resource version as well.

              * `type` (`pulumi.Input[str]`) - The type of processor. Valid Values: `Lambda`

          * `retryDuration` (`pulumi.Input[float]`) - The length of time during which Firehose retries delivery after a failure, starting from the initial request and including the first attempt. The default value is 3600 seconds (60 minutes). Firehose does not retry if the value of DurationInSeconds is 0 (zero) or if the first delivery attempt takes longer than the current value.
          * `role_arn` (`pulumi.Input[str]`) - The arn of the role the stream assumes.
          * `s3BackupConfiguration` (`pulumi.Input[dict]`) - The configuration for backup in Amazon S3. Required if `s3_backup_mode` is `Enabled`. Supports the same fields as `s3_configuration` object.
            * `bucketArn` (`pulumi.Input[str]`) - The ARN of the S3 bucket
            * `bufferInterval` (`pulumi.Input[float]`) - Buffer incoming data for the specified period of time, in seconds, before delivering it to the destination. The default value is 300.
            * `bufferSize` (`pulumi.Input[float]`) - Buffer incoming data to the specified size, in MBs, before delivering it to the destination. The default value is 5.
              We recommend setting SizeInMBs to a value greater than the amount of data you typically ingest into the delivery stream in 10 seconds. For example, if you typically ingest data at 1 MB/sec set SizeInMBs to be 10 MB or higher.
            * `cloudwatch_logging_options` (`pulumi.Input[dict]`) - The CloudWatch Logging Options for the delivery stream. More details are given below
              * `enabled` (`pulumi.Input[bool]`) - Enables or disables the logging. Defaults to `false`.
              * `log_group_name` (`pulumi.Input[str]`) - The CloudWatch group name for logging. This value is required if `enabled` is true.
              * `logStreamName` (`pulumi.Input[str]`) - The CloudWatch log stream name for logging. This value is required if `enabled` is true.

            * `compressionFormat` (`pulumi.Input[str]`) - The compression format. If no value is specified, the default is UNCOMPRESSED. Other supported values are GZIP, ZIP & Snappy. If the destination is redshift you cannot use ZIP or Snappy.
            * `kms_key_arn` (`pulumi.Input[str]`) - Specifies the KMS key ARN the stream will use to encrypt data. If not set, no encryption will
              be used.
            * `prefix` (`pulumi.Input[str]`) - The "YYYY/MM/DD/HH" time format prefix is automatically used for delivered S3 files. You can specify an extra prefix to be added in front of the time format prefix. Note that if the prefix ends with a slash, it appears as a folder in the S3 bucket
            * `role_arn` (`pulumi.Input[str]`) - The role that Kinesis Data Firehose can use to access AWS Glue. This role must be in the same account you use for Kinesis Data Firehose. Cross-account roles aren't allowed.

          * `s3BackupMode` (`pulumi.Input[str]`) - The Amazon S3 backup mode.  Valid values are `Disabled` and `Enabled`.  Default value is `Disabled`.
          * `username` (`pulumi.Input[str]`) - The username that the firehose delivery stream will assume. It is strongly recommended that the username and password provided is used exclusively for Amazon Kinesis Firehose purposes, and that the permissions for the account are restricted for Amazon Redshift INSERT permissions.

        The **s3_configuration** object supports the following:

          * `bucketArn` (`pulumi.Input[str]`) - The ARN of the S3 bucket
          * `bufferInterval` (`pulumi.Input[float]`) - Buffer incoming data for the specified period of time, in seconds, before delivering it to the destination. The default value is 300.
          * `bufferSize` (`pulumi.Input[float]`) - Buffer incoming data to the specified size, in MBs, before delivering it to the destination. The default value is 5.
            We recommend setting SizeInMBs to a value greater than the amount of data you typically ingest into the delivery stream in 10 seconds. For example, if you typically ingest data at 1 MB/sec set SizeInMBs to be 10 MB or higher.
          * `cloudwatch_logging_options` (`pulumi.Input[dict]`) - The CloudWatch Logging Options for the delivery stream. More details are given below
            * `enabled` (`pulumi.Input[bool]`) - Enables or disables the logging. Defaults to `false`.
            * `log_group_name` (`pulumi.Input[str]`) - The CloudWatch group name for logging. This value is required if `enabled` is true.
            * `logStreamName` (`pulumi.Input[str]`) - The CloudWatch log stream name for logging. This value is required if `enabled` is true.

          * `compressionFormat` (`pulumi.Input[str]`) - The compression format. If no value is specified, the default is UNCOMPRESSED. Other supported values are GZIP, ZIP & Snappy. If the destination is redshift you cannot use ZIP or Snappy.
          * `kms_key_arn` (`pulumi.Input[str]`) - Specifies the KMS key ARN the stream will use to encrypt data. If not set, no encryption will
            be used.
          * `prefix` (`pulumi.Input[str]`) - The "YYYY/MM/DD/HH" time format prefix is automatically used for delivered S3 files. You can specify an extra prefix to be added in front of the time format prefix. Note that if the prefix ends with a slash, it appears as a folder in the S3 bucket
          * `role_arn` (`pulumi.Input[str]`) - The role that Kinesis Data Firehose can use to access AWS Glue. This role must be in the same account you use for Kinesis Data Firehose. Cross-account roles aren't allowed.

        The **server_side_encryption** object supports the following:

          * `enabled` (`pulumi.Input[bool]`) - Whether to enable encryption at rest. Default is `false`.

        The **splunk_configuration** object supports the following:

          * `cloudwatch_logging_options` (`pulumi.Input[dict]`) - The CloudWatch Logging Options for the delivery stream. More details are given below.
            * `enabled` (`pulumi.Input[bool]`) - Enables or disables the logging. Defaults to `false`.
            * `log_group_name` (`pulumi.Input[str]`) - The CloudWatch group name for logging. This value is required if `enabled` is true.
            * `logStreamName` (`pulumi.Input[str]`) - The CloudWatch log stream name for logging. This value is required if `enabled` is true.

          * `hecAcknowledgmentTimeout` (`pulumi.Input[float]`) - The amount of time, in seconds between 180 and 600, that Kinesis Firehose waits to receive an acknowledgment from Splunk after it sends it data.
          * `hecEndpoint` (`pulumi.Input[str]`) - The HTTP Event Collector (HEC) endpoint to which Kinesis Firehose sends your data.
          * `hecEndpointType` (`pulumi.Input[str]`) - The HEC endpoint type. Valid values are `Raw` or `Event`. The default value is `Raw`.
          * `hecToken` (`pulumi.Input[str]`) - The GUID that you obtain from your Splunk cluster when you create a new HEC endpoint.
          * `processingConfiguration` (`pulumi.Input[dict]`) - The data processing configuration.  More details are given below.
            * `enabled` (`pulumi.Input[bool]`) - Enables or disables data processing.
            * `processors` (`pulumi.Input[list]`) - Array of data processors. More details are given below
              * `parameters` (`pulumi.Input[list]`) - Array of processor parameters. More details are given below
                * `parameterName` (`pulumi.Input[str]`) - Parameter name. Valid Values: `LambdaArn`, `NumberOfRetries`, `RoleArn`, `BufferSizeInMBs`, `BufferIntervalInSeconds`
                * `parameterValue` (`pulumi.Input[str]`) - Parameter value. Must be between 1 and 512 length (inclusive). When providing a Lambda ARN, you should specify the resource version as well.

              * `type` (`pulumi.Input[str]`) - The type of processor. Valid Values: `Lambda`

          * `retryDuration` (`pulumi.Input[float]`) - After an initial failure to deliver to Splunk, the total amount of time, in seconds between 0 to 7200, during which Firehose re-attempts delivery (including the first attempt).  After this time has elapsed, the failed documents are written to Amazon S3.  The default value is 300s.  There will be no retry if the value is 0.
          * `s3BackupMode` (`pulumi.Input[str]`) - Defines how documents should be delivered to Amazon S3.  Valid values are `FailedEventsOnly` and `AllEvents`.  Default value is `FailedEventsOnly`.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["arn"] = arn
        __props__["destination"] = destination
        __props__["destination_id"] = destination_id
        __props__["elasticsearch_configuration"] = elasticsearch_configuration
        __props__["extended_s3_configuration"] = extended_s3_configuration
        __props__["kinesis_source_configuration"] = kinesis_source_configuration
        __props__["name"] = name
        __props__["redshift_configuration"] = redshift_configuration
        __props__["s3_configuration"] = s3_configuration
        __props__["server_side_encryption"] = server_side_encryption
        __props__["splunk_configuration"] = splunk_configuration
        __props__["tags"] = tags
        __props__["version_id"] = version_id
        return FirehoseDeliveryStream(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class IdentityNotificationTopic(pulumi.CustomResource):
    identity: pulumi.Output[str]
    """
    The identity for which the Amazon SNS topic will be set. You can specify an identity by using its name or by using its Amazon Resource Name (ARN).
    """
    include_original_headers: pulumi.Output[bool]
    """
    Whether SES should include original email headers in SNS notifications of this type. *false* by default.
    """
    notification_type: pulumi.Output[str]
    """
    The type of notifications that will be published to the specified Amazon SNS topic. Valid Values: *Bounce*, *Complaint* or *Delivery*.
    """
    topic_arn: pulumi.Output[str]
    """
    The Amazon Resource Name (ARN) of the Amazon SNS topic. Can be set to "" (an empty string) to disable publishing.
    """
    def __init__(__self__, resource_name, opts=None, identity=None, include_original_headers=None, notification_type=None, topic_arn=None, __props__=None, __name__=None, __opts__=None):
        """
        Resource for managing SES Identity Notification Topics



        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] identity: The identity for which the Amazon SNS topic will be set. You can specify an identity by using its name or by using its Amazon Resource Name (ARN).
        :param pulumi.Input[bool] include_original_headers: Whether SES should include original email headers in SNS notifications of this type. *false* by default.
        :param pulumi.Input[str] notification_type: The type of notifications that will be published to the specified Amazon SNS topic. Valid Values: *Bounce*, *Complaint* or *Delivery*.
        :param pulumi.Input[str] topic_arn: The Amazon Resource Name (ARN) of the Amazon SNS topic. Can be set to "" (an empty string) to disable publishing.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            if identity is None:
                raise TypeError("Missing required property 'identity'")
            __props__['identity'] = identity
            __props__['include_original_headers'] = include_original_headers
            if notification_type is None:
                raise TypeError("Missing required property 'notification_type'")
            __props__['notification_type'] = notification_type
            __props__['topic_arn'] = topic_arn
        super(IdentityNotificationTopic, __self__).__init__(
            'aws:ses/identityNotificationTopic:IdentityNotificationTopic',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, identity=None, include_original_headers=None, notification_type=None, topic_arn=None):
        """
        Get an existing IdentityNotificationTopic resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] identity: The identity for which the Amazon SNS topic will be set. You can specify an identity by using its name or by using its Amazon Resource Name (ARN).
        :param pulumi.Input[bool] include_original_headers: Whether SES should include original email headers in SNS notifications of this type. *false* by default.
        :param pulumi.Input[str] notification_type: The type of notifications that will be published to the specified Amazon SNS topic. Valid Values: *Bounce*, *Complaint* or *Delivery*.
        :param pulumi.Input[str] topic_arn: The Amazon Resource Name (ARN) of the Amazon SNS topic. Can be set to "" (an empty string) to disable publishing.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["identity"] = identity
        __props__["include_original_headers"] = include_original_headers
        __props__["notification_type"] = notification_type
        __props__["topic_arn"] = topic_arn
        return IdentityNotificationTopic(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class BucketObject(pulumi.CustomResource):
    acl: pulumi.Output[str]
    """
    The [canned ACL](https://docs.aws.amazon.com/AmazonS3/latest/dev/acl-overview.html#canned-acl) to apply. Defaults to "private".
    """
    bucket: pulumi.Output[str]
    """
    The name of the bucket to put the file in. Alternatively, an [S3 access point](https://docs.aws.amazon.com/AmazonS3/latest/dev/using-access-points.html) ARN can be specified.
    """
    cache_control: pulumi.Output[str]
    """
    Specifies caching behavior along the request/reply chain Read [w3c cache_control](http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.9) for further details.
    """
    content: pulumi.Output[str]
    """
    Literal string value to use as the object content, which will be uploaded as UTF-8-encoded text.
    """
    content_base64: pulumi.Output[str]
    """
    Base64-encoded data that will be decoded and uploaded as raw bytes for the object content. This allows safely uploading non-UTF8 binary data, but is recommended only for small content such as the result of the `gzipbase64` function with small text strings. For larger objects, use `source` to stream the content from a disk file.
    """
    content_disposition: pulumi.Output[str]
    """
    Specifies presentational information for the object. Read [w3c content_disposition](http://www.w3.org/Protocols/rfc2616/rfc2616-sec19.html#sec19.5.1) for further information.
    """
    content_encoding: pulumi.Output[str]
    """
    Specifies what content encodings have been applied to the object and thus what decoding mechanisms must be applied to obtain the media-type referenced by the Content-Type header field. Read [w3c content encoding](http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.11) for further information.
    """
    content_language: pulumi.Output[str]
    """
    The language the content is in e.g. en-US or en-GB.
    """
    content_type: pulumi.Output[str]
    """
    A standard MIME type describing the format of the object data, e.g. application/octet-stream. All Valid MIME Types are valid for this input.
    """
    etag: pulumi.Output[str]
    """
    Used to trigger updates. The only meaningful value is `${filemd5("path/to/file")}` (this provider 0.11.12 or later) or `${md5(file("path/to/file"))}` (this provider 0.11.11 or earlier).
    This attribute is not compatible with KMS encryption, `kms_key_id` or `server_side_encryption = "aws:kms"`.
    """
    force_destroy: pulumi.Output[bool]
    """
    Allow the object to be deleted by removing any legal hold on any object version.
    Default is `false`. This value should be set to `true` only if the bucket has S3 object lock enabled.
    """
    key: pulumi.Output[str]
    """
    The name of the object once it is in the bucket.
    """
    kms_key_id: pulumi.Output[str]
    """
    Specifies the AWS KMS Key ARN to use for object encryption.
    This value is a fully qualified **ARN** of the KMS Key. If using `kms.Key`,
    use the exported `arn` attribute:
    `kms_key_id = "${aws_kms_key.foo.arn}"`
    """
    metadata: pulumi.Output[dict]
    """
    A map of keys/values to provision metadata (will be automatically prefixed by `x-amz-meta-`, note that only lowercase label are currently supported by the AWS Go API).
    """
    object_lock_legal_hold_status: pulumi.Output[str]
    """
    The [legal hold](https://docs.aws.amazon.com/AmazonS3/latest/dev/object-lock-overview.html#object-lock-legal-holds) status that you want to apply to the specified object. Valid values are `ON` and `OFF`.
    """
    object_lock_mode: pulumi.Output[str]
    """
    The object lock [retention mode](https://docs.aws.amazon.com/AmazonS3/latest/dev/object-lock-overview.html#object-lock-retention-modes) that you want to apply to this object. Valid values are `GOVERNANCE` and `COMPLIANCE`.
    """
    object_lock_retain_until_date: pulumi.Output[str]
    """
    The date and time, in [RFC3339 format](https://tools.ietf.org/html/rfc3339#section-5.8), when this object's object lock will [expire](https://docs.aws.amazon.com/AmazonS3/latest/dev/object-lock-overview.html#object-lock-retention-periods).
    """
    server_side_encryption: pulumi.Output[str]
    """
    Specifies server-side encryption of the object in S3. Valid values are "`AES256`" and "`aws:kms`".
    """
    source: pulumi.Output[Union[pulumi.Asset, pulumi.Archive]]
    """
    The path to a file that will be read and uploaded as raw bytes for the object content.
    """
    storage_class: pulumi.Output[str]
    """
    Specifies the desired [Storage Class](http://docs.aws.amazon.com/AmazonS3/latest/dev/storage-class-intro.html)
    for the object. Can be either "`STANDARD`", "`REDUCED_REDUNDANCY`", "`ONEZONE_IA`", "`INTELLIGENT_TIERING`", "`GLACIER`", "`DEEP_ARCHIVE`", or "`STANDARD_IA`". Defaults to "`STANDARD`".
    """
    tags: pulumi.Output[dict]
    """
    A map of tags to assign to the object.
    """
    version_id: pulumi.Output[str]
    """
    A unique version ID value for the object, if bucket versioning
    is enabled.
    """
    website_redirect: pulumi.Output[str]
    """
    Specifies a target URL for [website redirect](http://docs.aws.amazon.com/AmazonS3/latest/dev/how-to-page-redirect.html).
    """
    def __init__(__self__, resource_name, opts=None, acl=None, bucket=None, cache_control=None, content=None, content_base64=None, content_disposition=None, content_encoding=None, content_language=None, content_type=None, etag=None, force_destroy=None, key=None, kms_key_id=None, metadata=None, object_lock_legal_hold_status=None, object_lock_mode=None, object_lock_retain_until_date=None, server_side_encryption=None, source=None, storage_class=None, tags=None, website_redirect=None, __props__=None, __name__=None, __opts__=None):
        """
        Provides a S3 bucket object resource.



        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] acl: The [canned ACL](https://docs.aws.amazon.com/AmazonS3/latest/dev/acl-overview.html#canned-acl) to apply. Defaults to "private".
        :param pulumi.Input[dict] bucket: The name of the bucket to put the file in. Alternatively, an [S3 access point](https://docs.aws.amazon.com/AmazonS3/latest/dev/using-access-points.html) ARN can be specified.
        :param pulumi.Input[str] cache_control: Specifies caching behavior along the request/reply chain Read [w3c cache_control](http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.9) for further details.
        :param pulumi.Input[str] content: Literal string value to use as the object content, which will be uploaded as UTF-8-encoded text.
        :param pulumi.Input[str] content_base64: Base64-encoded data that will be decoded and uploaded as raw bytes for the object content. This allows safely uploading non-UTF8 binary data, but is recommended only for small content such as the result of the `gzipbase64` function with small text strings. For larger objects, use `source` to stream the content from a disk file.
        :param pulumi.Input[str] content_disposition: Specifies presentational information for the object. Read [w3c content_disposition](http://www.w3.org/Protocols/rfc2616/rfc2616-sec19.html#sec19.5.1) for further information.
        :param pulumi.Input[str] content_encoding: Specifies what content encodings have been applied to the object and thus what decoding mechanisms must be applied to obtain the media-type referenced by the Content-Type header field. Read [w3c content encoding](http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.11) for further information.
        :param pulumi.Input[str] content_language: The language the content is in e.g. en-US or en-GB.
        :param pulumi.Input[str] content_type: A standard MIME type describing the format of the object data, e.g. application/octet-stream. All Valid MIME Types are valid for this input.
        :param pulumi.Input[str] etag: Used to trigger updates. The only meaningful value is `${filemd5("path/to/file")}` (this provider 0.11.12 or later) or `${md5(file("path/to/file"))}` (this provider 0.11.11 or earlier).
               This attribute is not compatible with KMS encryption, `kms_key_id` or `server_side_encryption = "aws:kms"`.
        :param pulumi.Input[bool] force_destroy: Allow the object to be deleted by removing any legal hold on any object version.
               Default is `false`. This value should be set to `true` only if the bucket has S3 object lock enabled.
        :param pulumi.Input[str] key: The name of the object once it is in the bucket.
        :param pulumi.Input[str] kms_key_id: Specifies the AWS KMS Key ARN to use for object encryption.
               This value is a fully qualified **ARN** of the KMS Key. If using `kms.Key`,
               use the exported `arn` attribute:
               `kms_key_id = "${aws_kms_key.foo.arn}"`
        :param pulumi.Input[dict] metadata: A map of keys/values to provision metadata (will be automatically prefixed by `x-amz-meta-`, note that only lowercase label are currently supported by the AWS Go API).
        :param pulumi.Input[str] object_lock_legal_hold_status: The [legal hold](https://docs.aws.amazon.com/AmazonS3/latest/dev/object-lock-overview.html#object-lock-legal-holds) status that you want to apply to the specified object. Valid values are `ON` and `OFF`.
        :param pulumi.Input[str] object_lock_mode: The object lock [retention mode](https://docs.aws.amazon.com/AmazonS3/latest/dev/object-lock-overview.html#object-lock-retention-modes) that you want to apply to this object. Valid values are `GOVERNANCE` and `COMPLIANCE`.
        :param pulumi.Input[str] object_lock_retain_until_date: The date and time, in [RFC3339 format](https://tools.ietf.org/html/rfc3339#section-5.8), when this object's object lock will [expire](https://docs.aws.amazon.com/AmazonS3/latest/dev/object-lock-overview.html#object-lock-retention-periods).
        :param pulumi.Input[str] server_side_encryption: Specifies server-side encryption of the object in S3. Valid values are "`AES256`" and "`aws:kms`".
        :param pulumi.Input[Union[pulumi.Asset, pulumi.Archive]] source: The path to a file that will be read and uploaded as raw bytes for the object content.
        :param pulumi.Input[str] storage_class: Specifies the desired [Storage Class](http://docs.aws.amazon.com/AmazonS3/latest/dev/storage-class-intro.html)
               for the object. Can be either "`STANDARD`", "`REDUCED_REDUNDANCY`", "`ONEZONE_IA`", "`INTELLIGENT_TIERING`", "`GLACIER`", "`DEEP_ARCHIVE`", or "`STANDARD_IA`". Defaults to "`STANDARD`".
        :param pulumi.Input[dict] tags: A map of tags to assign to the object.
        :param pulumi.Input[str] website_redirect: Specifies a target URL for [website redirect](http://docs.aws.amazon.com/AmazonS3/latest/dev/how-to-page-redirect.html).
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['acl'] = acl
            if bucket is None:
                raise TypeError("Missing required property 'bucket'")
            __props__['bucket'] = bucket
            __props__['cache_control'] = cache_control
            __props__['content'] = content
            __props__['content_base64'] = content_base64
            __props__['content_disposition'] = content_disposition
            __props__['content_encoding'] = content_encoding
            __props__['content_language'] = content_language
            __props__['content_type'] = content_type
            __props__['etag'] = etag
            __props__['force_destroy'] = force_destroy
            __props__['key'] = key
            __props__['kms_key_id'] = kms_key_id
            __props__['metadata'] = metadata
            __props__['object_lock_legal_hold_status'] = object_lock_legal_hold_status
            __props__['object_lock_mode'] = object_lock_mode
            __props__['object_lock_retain_until_date'] = object_lock_retain_until_date
            __props__['server_side_encryption'] = server_side_encryption
            __props__['source'] = source
            __props__['storage_class'] = storage_class
            __props__['tags'] = tags
            __props__['website_redirect'] = website_redirect
            __props__['version_id'] = None
        super(BucketObject, __self__).__init__(
            'aws:s3/bucketObject:BucketObject',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, acl=None, bucket=None, cache_control=None, content=None, content_base64=None, content_disposition=None, content_encoding=None, content_language=None, content_type=None, etag=None, force_destroy=None, key=None, kms_key_id=None, metadata=None, object_lock_legal_hold_status=None, object_lock_mode=None, object_lock_retain_until_date=None, server_side_encryption=None, source=None, storage_class=None, tags=None, version_id=None, website_redirect=None):
        """
        Get an existing BucketObject resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] acl: The [canned ACL](https://docs.aws.amazon.com/AmazonS3/latest/dev/acl-overview.html#canned-acl) to apply. Defaults to "private".
        :param pulumi.Input[dict] bucket: The name of the bucket to put the file in. Alternatively, an [S3 access point](https://docs.aws.amazon.com/AmazonS3/latest/dev/using-access-points.html) ARN can be specified.
        :param pulumi.Input[str] cache_control: Specifies caching behavior along the request/reply chain Read [w3c cache_control](http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.9) for further details.
        :param pulumi.Input[str] content: Literal string value to use as the object content, which will be uploaded as UTF-8-encoded text.
        :param pulumi.Input[str] content_base64: Base64-encoded data that will be decoded and uploaded as raw bytes for the object content. This allows safely uploading non-UTF8 binary data, but is recommended only for small content such as the result of the `gzipbase64` function with small text strings. For larger objects, use `source` to stream the content from a disk file.
        :param pulumi.Input[str] content_disposition: Specifies presentational information for the object. Read [w3c content_disposition](http://www.w3.org/Protocols/rfc2616/rfc2616-sec19.html#sec19.5.1) for further information.
        :param pulumi.Input[str] content_encoding: Specifies what content encodings have been applied to the object and thus what decoding mechanisms must be applied to obtain the media-type referenced by the Content-Type header field. Read [w3c content encoding](http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.11) for further information.
        :param pulumi.Input[str] content_language: The language the content is in e.g. en-US or en-GB.
        :param pulumi.Input[str] content_type: A standard MIME type describing the format of the object data, e.g. application/octet-stream. All Valid MIME Types are valid for this input.
        :param pulumi.Input[str] etag: Used to trigger updates. The only meaningful value is `${filemd5("path/to/file")}` (this provider 0.11.12 or later) or `${md5(file("path/to/file"))}` (this provider 0.11.11 or earlier).
               This attribute is not compatible with KMS encryption, `kms_key_id` or `server_side_encryption = "aws:kms"`.
        :param pulumi.Input[bool] force_destroy: Allow the object to be deleted by removing any legal hold on any object version.
               Default is `false`. This value should be set to `true` only if the bucket has S3 object lock enabled.
        :param pulumi.Input[str] key: The name of the object once it is in the bucket.
        :param pulumi.Input[str] kms_key_id: Specifies the AWS KMS Key ARN to use for object encryption.
               This value is a fully qualified **ARN** of the KMS Key. If using `kms.Key`,
               use the exported `arn` attribute:
               `kms_key_id = "${aws_kms_key.foo.arn}"`
        :param pulumi.Input[dict] metadata: A map of keys/values to provision metadata (will be automatically prefixed by `x-amz-meta-`, note that only lowercase label are currently supported by the AWS Go API).
        :param pulumi.Input[str] object_lock_legal_hold_status: The [legal hold](https://docs.aws.amazon.com/AmazonS3/latest/dev/object-lock-overview.html#object-lock-legal-holds) status that you want to apply to the specified object. Valid values are `ON` and `OFF`.
        :param pulumi.Input[str] object_lock_mode: The object lock [retention mode](https://docs.aws.amazon.com/AmazonS3/latest/dev/object-lock-overview.html#object-lock-retention-modes) that you want to apply to this object. Valid values are `GOVERNANCE` and `COMPLIANCE`.
        :param pulumi.Input[str] object_lock_retain_until_date: The date and time, in [RFC3339 format](https://tools.ietf.org/html/rfc3339#section-5.8), when this object's object lock will [expire](https://docs.aws.amazon.com/AmazonS3/latest/dev/object-lock-overview.html#object-lock-retention-periods).
        :param pulumi.Input[str] server_side_encryption: Specifies server-side encryption of the object in S3. Valid values are "`AES256`" and "`aws:kms`".
        :param pulumi.Input[Union[pulumi.Asset, pulumi.Archive]] source: The path to a file that will be read and uploaded as raw bytes for the object content.
        :param pulumi.Input[str] storage_class: Specifies the desired [Storage Class](http://docs.aws.amazon.com/AmazonS3/latest/dev/storage-class-intro.html)
               for the object. Can be either "`STANDARD`", "`REDUCED_REDUNDANCY`", "`ONEZONE_IA`", "`INTELLIGENT_TIERING`", "`GLACIER`", "`DEEP_ARCHIVE`", or "`STANDARD_IA`". Defaults to "`STANDARD`".
        :param pulumi.Input[dict] tags: A map of tags to assign to the object.
        :param pulumi.Input[str] version_id: A unique version ID value for the object, if bucket versioning
               is enabled.
        :param pulumi.Input[str] website_redirect: Specifies a target URL for [website redirect](http://docs.aws.amazon.com/AmazonS3/latest/dev/how-to-page-redirect.html).
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["acl"] = acl
        __props__["bucket"] = bucket
        __props__["cache_control"] = cache_control
        __props__["content"] = content
        __props__["content_base64"] = content_base64
        __props__["content_disposition"] = content_disposition
        __props__["content_encoding"] = content_encoding
        __props__["content_language"] = content_language
        __props__["content_type"] = content_type
        __props__["etag"] = etag
        __props__["force_destroy"] = force_destroy
        __props__["key"] = key
        __props__["kms_key_id"] = kms_key_id
        __props__["metadata"] = metadata
        __props__["object_lock_legal_hold_status"] = object_lock_legal_hold_status
        __props__["object_lock_mode"] = object_lock_mode
        __props__["object_lock_retain_until_date"] = object_lock_retain_until_date
        __props__["server_side_encryption"] = server_side_encryption
        __props__["source"] = source
        __props__["storage_class"] = storage_class
        __props__["tags"] = tags
        __props__["version_id"] = version_id
        __props__["website_redirect"] = website_redirect
        return BucketObject(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Snapshot(pulumi.CustomResource):
    allocated_storage: pulumi.Output[float]
    """
    Specifies the allocated storage size in gigabytes (GB).
    """
    availability_zone: pulumi.Output[str]
    """
    Specifies the name of the Availability Zone the DB instance was located in at the time of the DB snapshot.
    """
    db_instance_identifier: pulumi.Output[str]
    """
    The DB Instance Identifier from which to take the snapshot.
    """
    db_snapshot_arn: pulumi.Output[str]
    """
    The Amazon Resource Name (ARN) for the DB snapshot.
    """
    db_snapshot_identifier: pulumi.Output[str]
    """
    The Identifier for the snapshot.
    """
    encrypted: pulumi.Output[bool]
    """
    Specifies whether the DB snapshot is encrypted.
    """
    engine: pulumi.Output[str]
    """
    Specifies the name of the database engine.
    """
    engine_version: pulumi.Output[str]
    """
    Specifies the version of the database engine.
    """
    iops: pulumi.Output[float]
    """
    Specifies the Provisioned IOPS (I/O operations per second) value of the DB instance at the time of the snapshot.
    """
    kms_key_id: pulumi.Output[str]
    """
    The ARN for the KMS encryption key.
    """
    license_model: pulumi.Output[str]
    """
    License model information for the restored DB instance.
    """
    option_group_name: pulumi.Output[str]
    """
    Provides the option group name for the DB snapshot.
    """
    port: pulumi.Output[float]
    snapshot_type: pulumi.Output[str]
    source_db_snapshot_identifier: pulumi.Output[str]
    """
    The DB snapshot Arn that the DB snapshot was copied from. It only has value in case of cross customer or cross region copy.
    """
    source_region: pulumi.Output[str]
    """
    The region that the DB snapshot was created in or copied from.
    """
    status: pulumi.Output[str]
    """
    Specifies the status of this DB snapshot.
    """
    storage_type: pulumi.Output[str]
    """
    Specifies the storage type associated with DB snapshot.
    """
    tags: pulumi.Output[dict]
    """
    Key-value map of resource tags
    """
    vpc_id: pulumi.Output[str]
    """
    Specifies the storage type associated with DB snapshot.
    """
    def __init__(__self__, resource_name, opts=None, db_instance_identifier=None, db_snapshot_identifier=None, tags=None, __props__=None, __name__=None, __opts__=None):
        """
        Manages a RDS database instance snapshot. For managing RDS database cluster snapshots, see the [`rds.ClusterSnapshot` resource](https://www.terraform.io/docs/providers/aws/r/db_cluster_snapshot.html).



        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] db_instance_identifier: The DB Instance Identifier from which to take the snapshot.
        :param pulumi.Input[str] db_snapshot_identifier: The Identifier for the snapshot.
        :param pulumi.Input[dict] tags: Key-value map of resource tags
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            if db_instance_identifier is None:
                raise TypeError("Missing required property 'db_instance_identifier'")
            __props__['db_instance_identifier'] = db_instance_identifier
            if db_snapshot_identifier is None:
                raise TypeError("Missing required property 'db_snapshot_identifier'")
            __props__['db_snapshot_identifier'] = db_snapshot_identifier
            __props__['tags'] = tags
            __props__['allocated_storage'] = None
            __props__['availability_zone'] = None
            __props__['db_snapshot_arn'] = None
            __props__['encrypted'] = None
            __props__['engine'] = None
            __props__['engine_version'] = None
            __props__['iops'] = None
            __props__['kms_key_id'] = None
            __props__['license_model'] = None
            __props__['option_group_name'] = None
            __props__['port'] = None
            __props__['snapshot_type'] = None
            __props__['source_db_snapshot_identifier'] = None
            __props__['source_region'] = None
            __props__['status'] = None
            __props__['storage_type'] = None
            __props__['vpc_id'] = None
        super(Snapshot, __self__).__init__(
            'aws:rds/snapshot:Snapshot',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, allocated_storage=None, availability_zone=None, db_instance_identifier=None, db_snapshot_arn=None, db_snapshot_identifier=None, encrypted=None, engine=None, engine_version=None, iops=None, kms_key_id=None, license_model=None, option_group_name=None, port=None, snapshot_type=None, source_db_snapshot_identifier=None, source_region=None, status=None, storage_type=None, tags=None, vpc_id=None):
        """
        Get an existing Snapshot resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[float] allocated_storage: Specifies the allocated storage size in gigabytes (GB).
        :param pulumi.Input[str] availability_zone: Specifies the name of the Availability Zone the DB instance was located in at the time of the DB snapshot.
        :param pulumi.Input[str] db_instance_identifier: The DB Instance Identifier from which to take the snapshot.
        :param pulumi.Input[str] db_snapshot_arn: The Amazon Resource Name (ARN) for the DB snapshot.
        :param pulumi.Input[str] db_snapshot_identifier: The Identifier for the snapshot.
        :param pulumi.Input[bool] encrypted: Specifies whether the DB snapshot is encrypted.
        :param pulumi.Input[str] engine: Specifies the name of the database engine.
        :param pulumi.Input[str] engine_version: Specifies the version of the database engine.
        :param pulumi.Input[float] iops: Specifies the Provisioned IOPS (I/O operations per second) value of the DB instance at the time of the snapshot.
        :param pulumi.Input[str] kms_key_id: The ARN for the KMS encryption key.
        :param pulumi.Input[str] license_model: License model information for the restored DB instance.
        :param pulumi.Input[str] option_group_name: Provides the option group name for the DB snapshot.
        :param pulumi.Input[str] source_db_snapshot_identifier: The DB snapshot Arn that the DB snapshot was copied from. It only has value in case of cross customer or cross region copy.
        :param pulumi.Input[str] source_region: The region that the DB snapshot was created in or copied from.
        :param pulumi.Input[str] status: Specifies the status of this DB snapshot.
        :param pulumi.Input[str] storage_type: Specifies the storage type associated with DB snapshot.
        :param pulumi.Input[dict] tags: Key-value map of resource tags
        :param pulumi.Input[str] vpc_id: Specifies the storage type associated with DB snapshot.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["allocated_storage"] = allocated_storage
        __props__["availability_zone"] = availability_zone
        __props__["db_instance_identifier"] = db_instance_identifier
        __props__["db_snapshot_arn"] = db_snapshot_arn
        __props__["db_snapshot_identifier"] = db_snapshot_identifier
        __props__["encrypted"] = encrypted
        __props__["engine"] = engine
        __props__["engine_version"] = engine_version
        __props__["iops"] = iops
        __props__["kms_key_id"] = kms_key_id
        __props__["license_model"] = license_model
        __props__["option_group_name"] = option_group_name
        __props__["port"] = port
        __props__["snapshot_type"] = snapshot_type
        __props__["source_db_snapshot_identifier"] = source_db_snapshot_identifier
        __props__["source_region"] = source_region
        __props__["status"] = status
        __props__["storage_type"] = storage_type
        __props__["tags"] = tags
        __props__["vpc_id"] = vpc_id
        return Snapshot(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class OptionGroup(pulumi.CustomResource):
    arn: pulumi.Output[str]
    """
    The ARN of the db option group.
    """
    engine_name: pulumi.Output[str]
    """
    Specifies the name of the engine that this option group should be associated with.
    """
    major_engine_version: pulumi.Output[str]
    """
    Specifies the major version of the engine that this option group should be associated with.
    """
    name: pulumi.Output[str]
    """
    The Name of the setting.
    """
    name_prefix: pulumi.Output[str]
    """
    Creates a unique name beginning with the specified prefix. Conflicts with `name`. Must be lowercase, to match as it is stored in AWS.
    """
    option_group_description: pulumi.Output[str]
    """
    The description of the option group. Defaults to "Managed by Pulumi".
    """
    options: pulumi.Output[list]
    """
    A list of Options to apply.

      * `dbSecurityGroupMemberships` (`list`) - A list of DB Security Groups for which the option is enabled.
      * `optionName` (`str`) - The Name of the Option (e.g. MEMCACHED).
      * `optionSettings` (`list`) - A list of option settings to apply.
        * `name` (`str`) - The Name of the setting.
        * `value` (`str`) - The Value of the setting.

      * `port` (`float`) - The Port number when connecting to the Option (e.g. 11211).
      * `version` (`str`) - The version of the option (e.g. 13.1.0.0).
      * `vpcSecurityGroupMemberships` (`list`) - A list of VPC Security Groups for which the option is enabled.
    """
    tags: pulumi.Output[dict]
    """
    A map of tags to assign to the resource.
    """
    def __init__(__self__, resource_name, opts=None, engine_name=None, major_engine_version=None, name=None, name_prefix=None, option_group_description=None, options=None, tags=None, __props__=None, __name__=None, __opts__=None):
        """
        Provides an RDS DB option group resource. Documentation of the available options for various RDS engines can be found at:

        * [MariaDB Options](https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/Appendix.MariaDB.Options.html)
        * [Microsoft SQL Server Options](https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/Appendix.SQLServer.Options.html)
        * [MySQL Options](https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/Appendix.MySQL.Options.html)
        * [Oracle Options](https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/Appendix.Oracle.Options.html)



        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] engine_name: Specifies the name of the engine that this option group should be associated with.
        :param pulumi.Input[str] major_engine_version: Specifies the major version of the engine that this option group should be associated with.
        :param pulumi.Input[str] name: The Name of the setting.
        :param pulumi.Input[str] name_prefix: Creates a unique name beginning with the specified prefix. Conflicts with `name`. Must be lowercase, to match as it is stored in AWS.
        :param pulumi.Input[str] option_group_description: The description of the option group. Defaults to "Managed by Pulumi".
        :param pulumi.Input[list] options: A list of Options to apply.
        :param pulumi.Input[dict] tags: A map of tags to assign to the resource.

        The **options** object supports the following:

          * `dbSecurityGroupMemberships` (`pulumi.Input[list]`) - A list of DB Security Groups for which the option is enabled.
          * `optionName` (`pulumi.Input[str]`) - The Name of the Option (e.g. MEMCACHED).
          * `optionSettings` (`pulumi.Input[list]`) - A list of option settings to apply.
            * `name` (`pulumi.Input[str]`) - The Name of the setting.
            * `value` (`pulumi.Input[str]`) - The Value of the setting.

          * `port` (`pulumi.Input[float]`) - The Port number when connecting to the Option (e.g. 11211).
          * `version` (`pulumi.Input[str]`) - The version of the option (e.g. 13.1.0.0).
          * `vpcSecurityGroupMemberships` (`pulumi.Input[list]`) - A list of VPC Security Groups for which the option is enabled.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            if engine_name is None:
                raise TypeError("Missing required property 'engine_name'")
            __props__['engine_name'] = engine_name
            if major_engine_version is None:
                raise TypeError("Missing required property 'major_engine_version'")
            __props__['major_engine_version'] = major_engine_version
            __props__['name'] = name
            __props__['name_prefix'] = name_prefix
            if option_group_description is None:
                option_group_description = 'Managed by Pulumi'
            __props__['option_group_description'] = option_group_description
            __props__['options'] = options
            __props__['tags'] = tags
            __props__['arn'] = None
        super(OptionGroup, __self__).__init__(
            'aws:rds/optionGroup:OptionGroup',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, arn=None, engine_name=None, major_engine_version=None, name=None, name_prefix=None, option_group_description=None, options=None, tags=None):
        """
        Get an existing OptionGroup resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] arn: The ARN of the db option group.
        :param pulumi.Input[str] engine_name: Specifies the name of the engine that this option group should be associated with.
        :param pulumi.Input[str] major_engine_version: Specifies the major version of the engine that this option group should be associated with.
        :param pulumi.Input[str] name: The Name of the setting.
        :param pulumi.Input[str] name_prefix: Creates a unique name beginning with the specified prefix. Conflicts with `name`. Must be lowercase, to match as it is stored in AWS.
        :param pulumi.Input[str] option_group_description: The description of the option group. Defaults to "Managed by Pulumi".
        :param pulumi.Input[list] options: A list of Options to apply.
        :param pulumi.Input[dict] tags: A map of tags to assign to the resource.

        The **options** object supports the following:

          * `dbSecurityGroupMemberships` (`pulumi.Input[list]`) - A list of DB Security Groups for which the option is enabled.
          * `optionName` (`pulumi.Input[str]`) - The Name of the Option (e.g. MEMCACHED).
          * `optionSettings` (`pulumi.Input[list]`) - A list of option settings to apply.
            * `name` (`pulumi.Input[str]`) - The Name of the setting.
            * `value` (`pulumi.Input[str]`) - The Value of the setting.

          * `port` (`pulumi.Input[float]`) - The Port number when connecting to the Option (e.g. 11211).
          * `version` (`pulumi.Input[str]`) - The version of the option (e.g. 13.1.0.0).
          * `vpcSecurityGroupMemberships` (`pulumi.Input[list]`) - A list of VPC Security Groups for which the option is enabled.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["arn"] = arn
        __props__["engine_name"] = engine_name
        __props__["major_engine_version"] = major_engine_version
        __props__["name"] = name
        __props__["name_prefix"] = name_prefix
        __props__["option_group_description"] = option_group_description
        __props__["options"] = options
        __props__["tags"] = tags
        return OptionGroup(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Route(pulumi.CustomResource):
    api_id: pulumi.Output[str]
    """
    The API identifier.
    """
    api_key_required: pulumi.Output[bool]
    """
    Boolean whether an API key is required for the route. Defaults to `false`.
    """
    authorization_scopes: pulumi.Output[list]
    """
    The authorization scopes supported by this route. The scopes are used with a JWT authorizer to authorize the method invocation.
    """
    authorization_type: pulumi.Output[str]
    """
    The authorization type for the route.
    For WebSocket APIs, valid values are `NONE` for open access, `AWS_IAM` for using AWS IAM permissions, and `CUSTOM` for using a Lambda authorizer.
    For HTTP APIs, valid values are `NONE` for open access, or `JWT` for using JSON Web Tokens.
    Defaults to `NONE`.
    """
    authorizer_id: pulumi.Output[str]
    """
    The identifier of the [`apigatewayv2.Authorizer`](https://www.terraform.io/docs/providers/aws/r/apigatewayv2_authorizer.html) resource to be associated with this route, if the authorizationType is `CUSTOM`.
    """
    model_selection_expression: pulumi.Output[str]
    """
    The [model selection expression](https://docs.aws.amazon.com/apigateway/latest/developerguide/apigateway-websocket-api-selection-expressions.html#apigateway-websocket-api-model-selection-expressions) for the route.
    """
    operation_name: pulumi.Output[str]
    """
    The operation name for the route.
    """
    request_models: pulumi.Output[dict]
    """
    The request models for the route.
    """
    route_key: pulumi.Output[str]
    """
    The route key for the route.
    """
    route_response_selection_expression: pulumi.Output[str]
    """
    The [route response selection expression](https://docs.aws.amazon.com/apigateway/latest/developerguide/apigateway-websocket-api-selection-expressions.html#apigateway-websocket-api-route-response-selection-expressions) for the route.
    """
    target: pulumi.Output[str]
    """
    The target for the route.
    """
    def __init__(__self__, resource_name, opts=None, api_id=None, api_key_required=None, authorization_scopes=None, authorization_type=None, authorizer_id=None, model_selection_expression=None, operation_name=None, request_models=None, route_key=None, route_response_selection_expression=None, target=None, __props__=None, __name__=None, __opts__=None):
        """
        Manages an Amazon API Gateway Version 2 route.
        More information can be found in the [Amazon API Gateway Developer Guide](https://docs.aws.amazon.com/apigateway/latest/developerguide/apigateway-websocket-api.html).



        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] api_id: The API identifier.
        :param pulumi.Input[bool] api_key_required: Boolean whether an API key is required for the route. Defaults to `false`.
        :param pulumi.Input[list] authorization_scopes: The authorization scopes supported by this route. The scopes are used with a JWT authorizer to authorize the method invocation.
        :param pulumi.Input[str] authorization_type: The authorization type for the route.
               For WebSocket APIs, valid values are `NONE` for open access, `AWS_IAM` for using AWS IAM permissions, and `CUSTOM` for using a Lambda authorizer.
               For HTTP APIs, valid values are `NONE` for open access, or `JWT` for using JSON Web Tokens.
               Defaults to `NONE`.
        :param pulumi.Input[str] authorizer_id: The identifier of the [`apigatewayv2.Authorizer`](https://www.terraform.io/docs/providers/aws/r/apigatewayv2_authorizer.html) resource to be associated with this route, if the authorizationType is `CUSTOM`.
        :param pulumi.Input[str] model_selection_expression: The [model selection expression](https://docs.aws.amazon.com/apigateway/latest/developerguide/apigateway-websocket-api-selection-expressions.html#apigateway-websocket-api-model-selection-expressions) for the route.
        :param pulumi.Input[str] operation_name: The operation name for the route.
        :param pulumi.Input[dict] request_models: The request models for the route.
        :param pulumi.Input[str] route_key: The route key for the route.
        :param pulumi.Input[str] route_response_selection_expression: The [route response selection expression](https://docs.aws.amazon.com/apigateway/latest/developerguide/apigateway-websocket-api-selection-expressions.html#apigateway-websocket-api-route-response-selection-expressions) for the route.
        :param pulumi.Input[str] target: The target for the route.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            if api_id is None:
                raise TypeError("Missing required property 'api_id'")
            __props__['api_id'] = api_id
            __props__['api_key_required'] = api_key_required
            __props__['authorization_scopes'] = authorization_scopes
            __props__['authorization_type'] = authorization_type
            __props__['authorizer_id'] = authorizer_id
            __props__['model_selection_expression'] = model_selection_expression
            __props__['operation_name'] = operation_name
            __props__['request_models'] = request_models
            if route_key is None:
                raise TypeError("Missing required property 'route_key'")
            __props__['route_key'] = route_key
            __props__['route_response_selection_expression'] = route_response_selection_expression
            __props__['target'] = target
        super(Route, __self__).__init__(
            'aws:apigatewayv2/route:Route',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, api_id=None, api_key_required=None, authorization_scopes=None, authorization_type=None, authorizer_id=None, model_selection_expression=None, operation_name=None, request_models=None, route_key=None, route_response_selection_expression=None, target=None):
        """
        Get an existing Route resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] api_id: The API identifier.
        :param pulumi.Input[bool] api_key_required: Boolean whether an API key is required for the route. Defaults to `false`.
        :param pulumi.Input[list] authorization_scopes: The authorization scopes supported by this route. The scopes are used with a JWT authorizer to authorize the method invocation.
        :param pulumi.Input[str] authorization_type: The authorization type for the route.
               For WebSocket APIs, valid values are `NONE` for open access, `AWS_IAM` for using AWS IAM permissions, and `CUSTOM` for using a Lambda authorizer.
               For HTTP APIs, valid values are `NONE` for open access, or `JWT` for using JSON Web Tokens.
               Defaults to `NONE`.
        :param pulumi.Input[str] authorizer_id: The identifier of the [`apigatewayv2.Authorizer`](https://www.terraform.io/docs/providers/aws/r/apigatewayv2_authorizer.html) resource to be associated with this route, if the authorizationType is `CUSTOM`.
        :param pulumi.Input[str] model_selection_expression: The [model selection expression](https://docs.aws.amazon.com/apigateway/latest/developerguide/apigateway-websocket-api-selection-expressions.html#apigateway-websocket-api-model-selection-expressions) for the route.
        :param pulumi.Input[str] operation_name: The operation name for the route.
        :param pulumi.Input[dict] request_models: The request models for the route.
        :param pulumi.Input[str] route_key: The route key for the route.
        :param pulumi.Input[str] route_response_selection_expression: The [route response selection expression](https://docs.aws.amazon.com/apigateway/latest/developerguide/apigateway-websocket-api-selection-expressions.html#apigateway-websocket-api-route-response-selection-expressions) for the route.
        :param pulumi.Input[str] target: The target for the route.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["api_id"] = api_id
        __props__["api_key_required"] = api_key_required
        __props__["authorization_scopes"] = authorization_scopes
        __props__["authorization_type"] = authorization_type
        __props__["authorizer_id"] = authorizer_id
        __props__["model_selection_expression"] = model_selection_expression
        __props__["operation_name"] = operation_name
        __props__["request_models"] = request_models
        __props__["route_key"] = route_key
        __props__["route_response_selection_expression"] = route_response_selection_expression
        __props__["target"] = target
        return Route(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import pulumi
import pulumi.runtime
from .. import utilities, tables

class Policy(pulumi.CustomResource):
    """
    Provides a [Data Lifecycle Manager (DLM) lifecycle policy](https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/snapshot-lifecycle.html) for managing snapshots.
    """
    def __init__(__self__, __name__, __opts__=None, description=None, execution_role_arn=None, policy_details=None, state=None):
        """Create a Policy resource with the given unique name, props, and options."""
        if not __name__:
            raise TypeError('Missing resource name argument (for URN creation)')
        if not isinstance(__name__, str):
            raise TypeError('Expected resource name to be a string')
        if __opts__ and not isinstance(__opts__, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')

        __props__ = dict()

        if not description:
            raise TypeError('Missing required property description')
        __props__['description'] = description

        if not execution_role_arn:
            raise TypeError('Missing required property execution_role_arn')
        __props__['execution_role_arn'] = execution_role_arn

        if not policy_details:
            raise TypeError('Missing required property policy_details')
        __props__['policy_details'] = policy_details

        __props__['state'] = state

        super(Policy, __self__).__init__(
            'aws:dlm/policy:Policy',
            __name__,
            __props__,
            __opts__)


    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class RdsDbInstance(pulumi.CustomResource):
    db_password: pulumi.Output[str]
    """
    A db password
    """
    db_user: pulumi.Output[str]
    """
    A db username
    """
    rds_db_instance_arn: pulumi.Output[str]
    """
    The db instance to register for this stack. Changing this will force a new resource.
    """
    stack_id: pulumi.Output[str]
    """
    The stack to register a db instance for. Changing this will force a new resource.
    """
    def __init__(__self__, resource_name, opts=None, db_password=None, db_user=None, rds_db_instance_arn=None, stack_id=None, __props__=None, __name__=None, __opts__=None):
        """
        Provides an OpsWorks RDS DB Instance resource.

        > **Note:** All arguments including the username and password will be stored in the raw state as plain-text.
        [Read more about sensitive data in state](https://www.terraform.io/docs/state/sensitive-data.html).



        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] db_password: A db password
        :param pulumi.Input[str] db_user: A db username
        :param pulumi.Input[str] rds_db_instance_arn: The db instance to register for this stack. Changing this will force a new resource.
        :param pulumi.Input[str] stack_id: The stack to register a db instance for. Changing this will force a new resource.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            if db_password is None:
                raise TypeError("Missing required property 'db_password'")
            __props__['db_password'] = db_password
            if db_user is None:
                raise TypeError("Missing required property 'db_user'")
            __props__['db_user'] = db_user
            if rds_db_instance_arn is None:
                raise TypeError("Missing required property 'rds_db_instance_arn'")
            __props__['rds_db_instance_arn'] = rds_db_instance_arn
            if stack_id is None:
                raise TypeError("Missing required property 'stack_id'")
            __props__['stack_id'] = stack_id
        super(RdsDbInstance, __self__).__init__(
            'aws:opsworks/rdsDbInstance:RdsDbInstance',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, db_password=None, db_user=None, rds_db_instance_arn=None, stack_id=None):
        """
        Get an existing RdsDbInstance resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] db_password: A db password
        :param pulumi.Input[str] db_user: A db username
        :param pulumi.Input[str] rds_db_instance_arn: The db instance to register for this stack. Changing this will force a new resource.
        :param pulumi.Input[str] stack_id: The stack to register a db instance for. Changing this will force a new resource.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["db_password"] = db_password
        __props__["db_user"] = db_user
        __props__["rds_db_instance_arn"] = rds_db_instance_arn
        __props__["stack_id"] = stack_id
        return RdsDbInstance(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop

